package com.starbuds.server.service.rtc.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.media.LiveRankItem;
import com.starbuds.server.common.pojo.message.im.rtc.RtcMvpChangeMsg;
import com.starbuds.server.common.pojo.rtc.RtcRankItem;
import com.starbuds.server.common.pojo.rtc.RtcRankService;
import com.starbuds.server.common.pojo.rtc.RtcRoomRankItem;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.rtc.dao.RtcBattleRoomInDAO;
import com.starbuds.server.service.rtc.dao.StatRtcRoomDAO;
import com.starbuds.server.service.rtc.provider.RtcRankProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

/**
 * RTC 排行服务实现
 *
 * @author Lay
 * @date 2020/7/14
 */
@Slf4j
@Service(version = "1.0.0")
public class RtcRankServiceImpl implements RtcRankService {

    @Resource
    StatRtcRoomDAO statRtcRoomDAO;

    @Resource
    RtcBattleRoomInDAO rtcBattleRoomInDAO;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Resource
    RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Resource
    RtcRankProvider rtcRankProvider;


    /**
     * 获取星钻日榜
     *
     * @param roomId
     * @param start
     * @param end
     * @param isPrevious
     * @return
     */
    @Override
    public List<LiveRankItem> getDiamondRankDay(Long roomId, long start, long end, boolean isPrevious) {
        String[] times = isPrevious ? rtcRankProvider.getPreviousRankTimes() : rtcRankProvider.getRankTimes();
        String redisKey = RedisRtcKeys.getRtcRankDiamondDay(roomId, times[0]);

        return rtcRankProvider.getRankItems(redisKey, start, end);
    }

    /**
     * 获取房间昨日钻石MVP
     *
     * @param roomId
     * @return
     */
    @Override
    public LiveRankItem getYesterdayDiamondRankMVP(Long roomId) {
        List<LiveRankItem> diamondRankDay = getDiamondRankDay(roomId, 0, 0, true);
        if (CollectionUtil.isNotEmpty(diamondRankDay)) {
            LiveRankItem liveRankItem = diamondRankDay.get(0);
            liveRankItem.setRole(rtcRoomService.getRoomRole(roomId, liveRankItem.getUserId()));
            return liveRankItem;
        }
        return null;
    }

    /**
     * 发送mvp改变的消息
     *
     * @param roomIds
     */
    @Override
    public void sendRtcMvpChangeMsg(List<Long> roomIds) {
        if (CollectionUtil.isEmpty(roomIds)) {
            return;
        }
        for (Long roomId : roomIds) {
            LiveRankItem rankMVP = getYesterdayDiamondRankMVP(roomId);
            if (rankMVP != null) {
                RtcMvpChangeMsg mvpChangeMsg = new RtcMvpChangeMsg();
                mvpChangeMsg.setUserId(rankMVP.getUserId());
                mvpChangeMsg.setUserName(rankMVP.getUserName());
                mvpChangeMsg.setUserAvatar(rankMVP.getUserAvatar());
                mvpChangeMsg.setRankNo(rankMVP.getRankNo());
                mvpChangeMsg.setAmount(rankMVP.getAmount());
                mvpChangeMsg.setWealthLevel(rankMVP.getWealthLevel());
                mvpChangeMsg.setIsOnline(rankMVP.getIsOnline());
                mvpChangeMsg.setAnimationFile(rankMVP.getAnimationFile());
                mvpChangeMsg.setRole(rankMVP.getRole());
                mvpChangeMsg.setRoomId(roomId);
                imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, mvpChangeMsg);
            }
        }
    }

    /**
     * 获取星钻周榜
     *
     * @param roomId
     * @param start
     * @param end
     * @param isPrevious
     * @return
     */
    @Override
    public List<LiveRankItem> getDiamondRankWeek(Long roomId, long start, long end, boolean isPrevious) {
        String[] times = isPrevious ? rtcRankProvider.getPreviousRankTimes() : rtcRankProvider.getRankTimes();
        String redisKey = RedisRtcKeys.getRtcRankDiamondWeek(roomId, times[1]);

        return rtcRankProvider.getRankItems(redisKey, start, end);
    }

    /**
     * 获取星钻月榜
     *
     * @param roomId
     * @param start
     * @param end
     * @param isPrevious
     * @return
     */
    @Override
    public List<LiveRankItem> getDiamondRankMonth(Long roomId, long start, long end, boolean isPrevious) {
        String[] times = isPrevious ? rtcRankProvider.getPreviousRankTimes() : rtcRankProvider.getRankTimes();
        String redisKey = RedisRtcKeys.getRtcRankDiamondMonth(roomId, times[2]);

        return rtcRankProvider.getRankItems(redisKey, start, end);
    }

    /**
     * 获取星钻日榜贡献值
     *
     * @param roomId
     * @param userId
     * @param isPrevious
     * @return
     */
    @Override
    public LiveRankItem getDiamondDayRankItem(Long roomId, Long userId, boolean isPrevious) {
        LiveRankItem item = new LiveRankItem();
        String[] times = isPrevious ? rtcRankProvider.getPreviousRankTimes() : rtcRankProvider.getRankTimes();
        String redisKey = RedisRtcKeys.getRtcRankDiamondDay(roomId, times[0]);
        Integer rankNo = Optional.ofNullable(redisService.zRevRank(redisKey, String.valueOf(userId))).map(Long::intValue).orElse(-1);
        Long amount = Optional.ofNullable(redisService.zScore(redisKey, String.valueOf(userId))).map(Convert::toLong).orElse(0L);
        List<UserCardVO> userCardVOList = userService.getNameAndAvatarByIds(Collections.singletonList(userId));
        if (!CollectionUtils.isEmpty(userCardVOList)) {
            UserCardVO userCardVO = userCardVOList.get(0);
            item.setUserName(userCardVO.getUserName());
            item.setUserAvatar(userCardVO.getUserAvatar());
            item.setWealthLevel(userCardVO.getWealthLevel());
        }
        item.setAmount(amount);
        item.setRankNo(rankNo + 1);
        item.setUserId(userId);
        return item;
    }

    /**
     * 获取星钻周榜贡献值
     *
     * @param roomId
     * @param userId
     * @param isPrevious
     * @return
     */
    @Override
    public LiveRankItem getDiamondWeekRankItem(Long roomId, Long userId, boolean isPrevious) {
        LiveRankItem item = new LiveRankItem();
        String[] times = isPrevious ? rtcRankProvider.getPreviousRankTimes() : rtcRankProvider.getRankTimes();
        String redisKey = RedisRtcKeys.getRtcRankDiamondWeek(roomId, times[1]);
        Integer rankNo = Optional.ofNullable(redisService.zRevRank(redisKey, String.valueOf(userId))).map(Long::intValue).orElse(-1);
        Long amount = Optional.ofNullable(redisService.zScore(redisKey, String.valueOf(userId))).map(Convert::toLong).orElse(0L);
        List<UserCardVO> userCardVOList = userService.getNameAndAvatarByIds(Collections.singletonList(userId));
        if (!CollectionUtils.isEmpty(userCardVOList)) {
            UserCardVO userCardVO = userCardVOList.get(0);
            item.setUserName(userCardVO.getUserName());
            item.setUserAvatar(userCardVO.getUserAvatar());
            item.setWealthLevel(userCardVO.getWealthLevel());
        }
        item.setAmount(amount);
        item.setRankNo(rankNo + 1);
        item.setUserId(userId);
        return item;
    }

    /**
     * 获取星钻月榜贡献值
     *
     * @param roomId
     * @param userId
     * @param isPrevious
     * @return
     */
    @Override
    public LiveRankItem getDiamondMonthRankItem(Long roomId, Long userId, boolean isPrevious) {
        LiveRankItem item = new LiveRankItem();
        String[] times = isPrevious ? rtcRankProvider.getPreviousRankTimes() : rtcRankProvider.getRankTimes();
        String redisKey = RedisRtcKeys.getRtcRankDiamondMonth(roomId, times[2]);
        Integer rankNo = Optional.ofNullable(redisService.zRevRank(redisKey, String.valueOf(userId))).map(Long::intValue).orElse(-1);
        Long amount = Optional.ofNullable(redisService.zScore(redisKey, String.valueOf(userId))).map(Convert::toLong).orElse(0L);
        List<UserCardVO> userCardVOList = userService.getNameAndAvatarByIds(Collections.singletonList(userId));
        if (!CollectionUtils.isEmpty(userCardVOList)) {
            UserCardVO userCardVO = userCardVOList.get(0);
            item.setUserName(userCardVO.getUserName());
            item.setUserAvatar(userCardVO.getUserAvatar());
            item.setWealthLevel(userCardVO.getWealthLevel());
        }
        item.setAmount(amount);
        item.setRankNo(rankNo + 1);
        item.setUserId(userId);
        return item;
    }

    /**
     * 获取星尘日榜
     *
     * @param roomId
     * @param start
     * @param end
     * @param isPrevious
     * @return
     */
    @Override
    public List<LiveRankItem> getDustRankDay(Long roomId, long start, long end, boolean isPrevious) {
        String[] times = isPrevious ? rtcRankProvider.getPreviousRankTimes() : rtcRankProvider.getRankTimes();
        String redisKey = RedisRtcKeys.getRtcRankDustDay(roomId, times[0]);

        return rtcRankProvider.getRankItems(redisKey, start, end);
    }

    /**
     * 获取星尘周榜
     *
     * @param roomId
     * @param start
     * @param end
     * @param isPrevious
     * @return
     */
    @Override
    public List<LiveRankItem> getDustRankWeek(Long roomId, long start, long end, boolean isPrevious) {
        String[] times = isPrevious ? rtcRankProvider.getPreviousRankTimes() : rtcRankProvider.getRankTimes();
        String redisKey = RedisRtcKeys.getRtcRankDustWeek(roomId, times[1]);

        return rtcRankProvider.getRankItems(redisKey, start, end);
    }

    /**
     * 获取星尘月榜
     *
     * @param roomId
     * @param start
     * @param end
     * @param isPrevious
     * @return
     */
    @Override
    public List<LiveRankItem> getDustRankMonth(Long roomId, long start, long end, boolean isPrevious) {
        String[] times = isPrevious ? rtcRankProvider.getPreviousRankTimes() : rtcRankProvider.getRankTimes();
        String redisKey = RedisRtcKeys.getRtcRankDustMonth(roomId, times[2]);

        return rtcRankProvider.getRankItems(redisKey, start, end);
    }

    /**
     * 获取星尘日榜贡献值
     *
     * @param roomId
     * @param userId
     * @param isPrevious
     * @return
     */
    @Override
    public LiveRankItem getDustDayRankItem(Long roomId, Long userId, boolean isPrevious) {
        LiveRankItem item = new LiveRankItem();
        String[] times = isPrevious ? rtcRankProvider.getPreviousRankTimes() : rtcRankProvider.getRankTimes();
        String redisKey = RedisRtcKeys.getRtcRankDustDay(roomId, times[0]);
        Integer rankNo = Optional.ofNullable(redisService.zRevRank(redisKey, String.valueOf(userId))).map(Long::intValue).orElse(-1);
        Long amount = Optional.ofNullable(redisService.zScore(redisKey, String.valueOf(userId))).map(Convert::toLong).orElse(0L);
        List<UserCardVO> userCardVOList = userService.getNameAndAvatarByIds(Collections.singletonList(userId));
        if (!CollectionUtils.isEmpty(userCardVOList)) {
            UserCardVO userCardVO = userCardVOList.get(0);
            item.setUserName(userCardVO.getUserName());
            item.setUserAvatar(userCardVO.getUserAvatar());
            item.setWealthLevel(userCardVO.getWealthLevel());
        }
        item.setAmount(amount);
        item.setRankNo(rankNo + 1);
        item.setUserId(userId);
        return item;
    }

    /**
     * 获取星尘周榜贡献值
     *
     * @param roomId
     * @param userId
     * @param isPrevious
     * @return
     */
    @Override
    public LiveRankItem getDustWeekRankItem(Long roomId, Long userId, boolean isPrevious) {
        LiveRankItem item = new LiveRankItem();
        String[] times = isPrevious ? rtcRankProvider.getPreviousRankTimes() : rtcRankProvider.getRankTimes();
        String redisKey = RedisRtcKeys.getRtcRankDustWeek(roomId, times[1]);
        Integer rankNo = Optional.ofNullable(redisService.zRevRank(redisKey, String.valueOf(userId))).map(Long::intValue).orElse(-1);
        Long amount = Optional.ofNullable(redisService.zScore(redisKey, String.valueOf(userId))).map(Convert::toLong).orElse(0L);
        List<UserCardVO> userCardVOList = userService.getNameAndAvatarByIds(Collections.singletonList(userId));
        if (!CollectionUtils.isEmpty(userCardVOList)) {
            UserCardVO userCardVO = userCardVOList.get(0);
            item.setUserName(userCardVO.getUserName());
            item.setUserAvatar(userCardVO.getUserAvatar());
            item.setWealthLevel(userCardVO.getWealthLevel());
        }
        item.setAmount(amount);
        item.setRankNo(rankNo + 1);
        item.setUserId(userId);
        return item;
    }

    /**
     * 获取星尘月榜贡献值
     *
     * @param roomId
     * @param userId
     * @param isPrevious
     * @return
     */
    @Override
    public LiveRankItem getDustMonthRankItem(Long roomId, Long userId, boolean isPrevious) {
        LiveRankItem item = new LiveRankItem();
        String[] times = isPrevious ? rtcRankProvider.getPreviousRankTimes() : rtcRankProvider.getRankTimes();
        String redisKey = RedisRtcKeys.getRtcRankDustMonth(roomId, times[2]);
        Integer rankNo = Optional.ofNullable(redisService.zRevRank(redisKey, String.valueOf(userId))).map(Long::intValue).orElse(-1);
        Long amount = Optional.ofNullable(redisService.zScore(redisKey, String.valueOf(userId))).map(Convert::toLong).orElse(0L);
        List<UserCardVO> userCardVOList = userService.getNameAndAvatarByIds(Collections.singletonList(userId));
        if (!CollectionUtils.isEmpty(userCardVOList)) {
            UserCardVO userCardVO = userCardVOList.get(0);
            item.setUserName(userCardVO.getUserName());
            item.setUserAvatar(userCardVO.getUserAvatar());
            item.setWealthLevel(userCardVO.getWealthLevel());
        }
        item.setAmount(amount);
        item.setRankNo(rankNo + 1);
        item.setUserId(userId);
        return item;
    }

    /**
     * 获取语音房小时榜
     *
     * @param statTime
     * @return
     */
    @Override
    public List<RtcRankItem> getRtcHourRank(Long statTime) {
        return rtcRankProvider.getRtcHourRank(statTime);
    }

    /**
     * 获取语音房小时榜
     *
     * @param statTime
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @Override
    public PageList<RtcRankItem> getRtcHourRankPageList(Long statTime, int pageIndex, int pageSize) {
        PageList<RtcRankItem> page = new PageList<>(pageIndex, pageSize);
        List<RtcRankItem> rtcRankItems = statRtcRoomDAO.selectRtcHourRankPageList(statTime, page.getPageCursor(), pageSize);
        for (int i = 0; i < rtcRankItems.size(); i++) {
            RtcRankItem rtcRankItem = rtcRankItems.get(i);
            rtcRankItem.setRankNo((i + 1) + page.getPageCursor());
        }
        page.setList(rtcRankItems);
        page.setTotal(statRtcRoomDAO.countRtcHourRankPageList(statTime));
        return page;
    }

    /**
     * 获取小时榜榜首
     *
     * @param statTime
     * @return
     */
    @Override
    public RtcRankItem getRtcHourRankWinner(Long statTime) {
        return statRtcRoomDAO.getRtcHourRankWinner(statTime);
    }

    /**
     * 获取pk擂台榜
     *
     * @param startTime
     * @return endTime
     */
    @Override
    public List<RtcRoomRankItem> getRtcBattleRank(Long startTime, Long endTime) {
        return rtcBattleRoomInDAO.selectRtcBattleRank(startTime, endTime);
    }
}
