package com.starbuds.server.service.rtc.provider;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ArrayUtil;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.media.BattleStatusEnum;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rtc.RtcRoomBattleType;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.pojo.db.RtcBattleRoomInUserScoreDO;
import com.starbuds.server.common.pojo.db.RtcBattleRoomInVipScoreDO;
import com.starbuds.server.common.pojo.message.im.rtc.RtcBattleRoomInScoreChangeMsg;
import com.starbuds.server.common.pojo.message.im.rtc.RtcBattleRoomInStatusMsg;
import com.starbuds.server.common.pojo.rtc.RtcBattleStatus;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.service.rtc.dao.RtcBattleRoomInDAO;
import com.starbuds.server.service.rtc.dao.RtcBattleRoomInUserScoreDAO;
import com.starbuds.server.service.rtc.dao.RtcBattleRoomInVipScoreDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 房间内 PK 服务
 *
 * @author Lay
 * @date 2021/5/11
 */
@Component
@Slf4j
public class RtcBattleRoomInProvider {

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Resource
    RtcBattleRoomInDAO rtcBattleRoomInDAO;

    @Resource
    RtcBattleRoomInUserScoreDAO rtcBattleRoomInUserScoreDAO;

    @Resource
    RtcBattleRoomInVipScoreDAO rtcBattleRoomInVipScoreDAO;

    @Resource
    RtcBattleRoomPropertyProvider rtcBattleRoomPropertyProvider;

    /**
     * 左侧阵营
     */
    private static final int[] LEFT_CAMP_SEATS = new int[]{1, 2, 5, 6};

    /**
     * 右侧阵营
     */
    private static final int[] RIGHT_CAMP_SEATS = new int[]{3, 4, 7, 8};

    /**
     * 左侧阵营
     */
    private static final int LEFT_CAMP = -1;

    /**
     * 右侧阵营
     */
    private static final int RIGHT_CAMP = 1;

    /**
     * 增加积分
     *
     * @param roomId
     * @param battleId
     * @param vipUserId
     * @param targetMap
     * @param scoreValue
     */
    public void batchAddScore(Long roomId, Long battleId, Long vipUserId, Map<Long, Integer> targetMap, long scoreValue, long flowValue) {
        String battleStatusKey = RedisRtcKeys.getRtcBattleStatus(roomId);
        if (scoreValue < 1 || !redisService.hasKey(battleStatusKey)) {
            return;
        }

        Long leftCampScore = null;
        Long rightCampScore = null;
        // 红蓝流水
        Long leftCampFlow = null;
        Long rightCampFlow = null;

        Long[] leftCampVip = null;
        Long[] rightCampVip = null;
        List<RtcBattleRoomInScoreChangeMsg.CampUserScore> campUserScores = new ArrayList<>();

        for (Map.Entry<Long, Integer> entry : targetMap.entrySet()) {
            int camp = getCamp(entry.getValue());
            if (camp != 0) {
                if (LEFT_CAMP == camp) {
                    leftCampScore = Optional.ofNullable(leftCampScore).orElse(0L) + scoreValue;
                    leftCampFlow = Optional.ofNullable(leftCampFlow).orElse(0L) + flowValue;
                } else if (RIGHT_CAMP == camp) {
                    rightCampScore = Optional.ofNullable(rightCampScore).orElse(0L) + scoreValue;
                    rightCampFlow = Optional.ofNullable(rightCampFlow).orElse(0L) + flowValue;
                }

                // 增加用户流水
                addUserFlow(battleId, entry.getKey(), camp, flowValue);
                // 增加用户积分
                long newScore = addUserScore(battleId, entry.getKey(), camp, scoreValue);

                if (newScore > 0) {
                    campUserScores.add(new RtcBattleRoomInScoreChangeMsg.CampUserScore(entry.getKey(), newScore, camp));
                }
            }
        }

        // 增加阵营积分
        String campKey = RedisRtcKeys.getRtcBattleRoomInCampScore(battleId);
        // 增加阵营流水
        String flowKey = RedisRtcKeys.getRtcBattleRoomInCampFlow(battleId);

        if (leftCampScore != null && leftCampScore > 0) {
            String vipKey = RedisRtcKeys.getRtcBattleRoomInCampVipScore(battleId, LEFT_CAMP);
            redisService.zIncrementScore(vipKey, String.valueOf(vipUserId), leftCampScore);

            // 增加阵营积分并赋值
            leftCampScore = redisService.hashIncreaseLong(campKey, String.valueOf(LEFT_CAMP), leftCampScore);

            // 增加阵营流水
            redisService.hashIncreaseLong(flowKey, String.valueOf(LEFT_CAMP), leftCampFlow);

            // VIP 处理
            Set<Tuple> vipSet = redisService.zReverseRangeWithScores(vipKey, 0, 3);
            if (!CollectionUtils.isEmpty(vipSet)) {
                List<Tuple> vipList = new ArrayList<>(vipSet);
                vipList.sort(Comparator.comparingDouble(Tuple::getScore).reversed());
                leftCampVip = new Long[vipList.size()];
                int index = 0;
                for (Tuple tuple : vipList) {
                    leftCampVip[index] = Convert.toLong(tuple.getElement());
                    index++;
                }
            }
        }

        if (rightCampScore != null && rightCampScore > 0) {
            String vipKey = RedisRtcKeys.getRtcBattleRoomInCampVipScore(battleId, RIGHT_CAMP);
            redisService.zIncrementScore(vipKey, String.valueOf(vipUserId), rightCampScore);

            // 增加阵营积分并赋总值
            rightCampScore = redisService.hashIncreaseLong(campKey, String.valueOf(RIGHT_CAMP), rightCampScore);

            // 增加阵营流水
            redisService.hashIncreaseLong(flowKey, String.valueOf(RIGHT_CAMP), rightCampFlow);

            // VIP 处理
            Set<Tuple> vipSet = redisService.zReverseRangeWithScores(vipKey, 0, 3);
            List<Tuple> vipList = new ArrayList<>(vipSet);
            vipList.sort(Comparator.comparingDouble(Tuple::getScore).reversed());
            rightCampVip = new Long[vipList.size()];
            int index = 0;
            for (Tuple tuple : vipList) {
                rightCampVip[index] = Convert.toLong(tuple.getElement());
                index++;
            }
        }

        // 同步积分 和 VIP
        rtcBattleRoomPropertyProvider.setCampScoreAndCampVip(roomId, leftCampScore, rightCampScore, leftCampVip, rightCampVip);

        // 同步流水
        rtcBattleRoomPropertyProvider.setCampScoreAndCampVip(roomId, leftCampScore, rightCampScore, leftCampVip, rightCampVip);

        // 积分变化消息
        RtcBattleRoomInScoreChangeMsg scoreChangeMsg = new RtcBattleRoomInScoreChangeMsg();
        scoreChangeMsg.setUserScoreList(campUserScores);
        imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, scoreChangeMsg);
    }

    /**
     * 增加个人积分
     */
    private long addUserScore(Long battleId, Long userId, int camp, long scoreValue) {
        try {
            String campUserKey = RedisRtcKeys.getRtcBattleRoomInCampUserScore(battleId, camp);
            return Convert.toLong(redisService.zIncrementScore(campUserKey, String.valueOf(userId), scoreValue), 0L);
        } catch (Exception e) {
            log.error("个人积分增加失败", e);
        }
        return 0;
    }

    /**
     * 增加个人流水
     */
    private long addUserFlow(Long battleId, Long userId, int camp, long flowValue) {
        try {
            String campUserKey = RedisRtcKeys.getRtcBattleRoomInCampUserFlow(battleId, camp);
            return Convert.toLong(redisService.zIncrementScore(campUserKey, String.valueOf(userId), flowValue), 0L);
        } catch (Exception e) {
            log.error("个人积分增加失败", e);
        }
        return 0;
    }

    public Map<Long, Long> getBattleScoreMap(Long battleId, Map<Integer, Long> seatUserMap) {
        String leftCampKey = RedisRtcKeys.getRtcBattleRoomInCampUserScore(battleId, LEFT_CAMP);
        String rightCampKey = RedisRtcKeys.getRtcBattleRoomInCampUserScore(battleId, RIGHT_CAMP);

        Set<Tuple> leftCampScoreSet = redisService.zReverseRangeWithScores(leftCampKey, 0, -1);
        Set<Tuple> rightCampScoreSet = redisService.zReverseRangeWithScores(rightCampKey, 0, -1);
        Map<Long, Long> leftCampScoreMap;
        Map<Long, Long> rightCampScoreMap;

        // 数据构造
        if (!CollectionUtils.isEmpty(leftCampScoreSet)) {
            leftCampScoreMap = leftCampScoreSet.stream().collect(Collectors.toMap(t -> Convert.toLong(t.getElement()), t -> Convert.toLong(t.getScore(), 0L)));
        } else {
            leftCampScoreMap = new HashMap<>();
        }

        if (!CollectionUtils.isEmpty(rightCampScoreSet)) {
            rightCampScoreMap = rightCampScoreSet.stream().collect(Collectors.toMap(t -> Convert.toLong(t.getElement()), t -> Convert.toLong(t.getScore(), 0L)));
        } else {
            rightCampScoreMap = new HashMap<>();
        }

        Map<Long, Long> scoreMap = new HashMap<>();
        for (Map.Entry<Integer, Long> entry : seatUserMap.entrySet()) {
            int camp = getCamp(entry.getKey());
            Long userId = entry.getValue();
            if (LEFT_CAMP == camp) {
                scoreMap.put(userId, leftCampScoreMap.getOrDefault(userId, 0L));
            } else if (RIGHT_CAMP == camp) {
                scoreMap.put(userId, rightCampScoreMap.getOrDefault(userId, 0L));
            }
        }

        return scoreMap;
    }

    /**
     * 获取 PK 标识
     *
     * @param roomId
     * @return
     */
    public Long getBattleId(Long roomId) {
        String battleStatusKey = RedisRtcKeys.getRtcBattleStatus(roomId);
        String status = redisService.get(battleStatusKey);
        if (StringUtils.isNotBlank(status)) {
            RtcBattleStatus battleStatus = new RtcBattleStatus(status);
            if (RtcRoomBattleType.ROOM_IN == battleStatus.getBattleType()) {
                return battleStatus.getBattleId();
            }
        }
        return null;
    }

    /**
     * PK 结束
     *
     * @param roomId
     * @param battleId
     */
    public boolean battleOver(Long roomId, Long battleId) {
        String battleStatusKey = RedisRtcKeys.getRtcBattleStatus(roomId);

        // 更新数据库
        if (rtcBattleRoomInDAO.updateFinished(battleId, BattleStatusEnum.Over.getValue(), System.currentTimeMillis(), BattleStatusEnum.Battling.getValue()) > 0) {
            // 移除缓存
            redisService.delKey(battleStatusKey);

            // 结算并发送消息
            String campKey = RedisRtcKeys.getRtcBattleRoomInCampScore(battleId);
            String flowKey = RedisRtcKeys.getRtcBattleRoomInCampFlow(battleId);
            Map<Object, Object> campScoreMap = redisService.hashGetAll(campKey);
            Map<Object, Object> campFlowMap = redisService.hashGetAll(flowKey);
            long leftCampScore = Convert.toLong(campScoreMap.get(String.valueOf(LEFT_CAMP)), 0L);
            long rightCampScore = Convert.toLong(campScoreMap.get(String.valueOf(RIGHT_CAMP)), 0L);

            long leftCampFlow = Convert.toLong(campFlowMap.get(String.valueOf(LEFT_CAMP)), 0L);
            long rightCampFlow = Convert.toLong(campFlowMap.get(String.valueOf(RIGHT_CAMP)), 0L);

            // PK 结果
            int battleResult;
            if (leftCampScore > rightCampScore) {
                battleResult = LEFT_CAMP;
            } else if (rightCampScore > leftCampScore) {
                battleResult = RIGHT_CAMP;
            } else {
                battleResult = 0;
            }

            // 更新结果
            rtcBattleRoomInDAO.updateResult(battleId, battleResult, leftCampScore, rightCampScore, leftCampFlow, rightCampFlow);

            // 拉取所有收益
            String leftUserKey = RedisRtcKeys.getRtcBattleRoomInCampUserScore(battleId, LEFT_CAMP);
            Set<Tuple> leftUserSet = redisService.zReverseRangeWithScores(leftUserKey, 0, -1);
            String rightUserKey = RedisRtcKeys.getRtcBattleRoomInCampUserScore(battleId, RIGHT_CAMP);
            Set<Tuple> rightUserSet = redisService.zReverseRangeWithScores(rightUserKey, 0, -1);

            // 拉取所有流水
            String leftUserFlowKey = RedisRtcKeys.getRtcBattleRoomInCampUserFlow(battleId, LEFT_CAMP);
            Set<Tuple> leftUserFlowSet = redisService.zReverseRangeWithScores(leftUserFlowKey, 0, -1);

            String rightUserFlowKey = RedisRtcKeys.getRtcBattleRoomInCampUserFlow(battleId, RIGHT_CAMP);
            Set<Tuple> rightUserFlowSet = redisService.zReverseRangeWithScores(rightUserFlowKey, 0, -1);

            // 持久化
            List<RtcBattleRoomInUserScoreDO> userScoreDOList = new ArrayList<>();
            if (!CollectionUtils.isEmpty(leftUserSet)) {
                for (Tuple tuple : leftUserSet) {
                    RtcBattleRoomInUserScoreDO userScoreDO = new RtcBattleRoomInUserScoreDO();
                    userScoreDO.setBattleId(battleId);
                    userScoreDO.setUserId(Convert.toLong(tuple.getElement()));
                    userScoreDO.setCamp(LEFT_CAMP);
                    userScoreDO.setScore(Convert.toLong(tuple.getScore()));
                    Double flow = leftUserFlowSet.stream().filter(Objects::nonNull).filter(s -> s.getElement().equals(tuple.getElement())).findFirst().orElse(new Tuple()).getScore();
                    userScoreDO.setFlow(flow.longValue());
                    userScoreDOList.add(userScoreDO);
                }
            }
            if (!CollectionUtils.isEmpty(rightUserSet)) {
                for (Tuple tuple : rightUserSet) {
                    RtcBattleRoomInUserScoreDO userScoreDO = new RtcBattleRoomInUserScoreDO();
                    userScoreDO.setBattleId(battleId);
                    userScoreDO.setUserId(Convert.toLong(tuple.getElement()));
                    userScoreDO.setCamp(RIGHT_CAMP);
                    userScoreDO.setScore(Convert.toLong(tuple.getScore()));
                    Double flow = rightUserFlowSet.stream().filter(Objects::nonNull).filter(s -> s.getElement().equals(tuple.getElement())).findFirst().orElse(new Tuple()).getScore();
                    userScoreDO.setFlow(flow.longValue());
                    userScoreDOList.add(userScoreDO);
                }
            }

            if (!CollectionUtils.isEmpty(userScoreDOList)) {
                rtcBattleRoomInUserScoreDAO.batchInsert(userScoreDOList);
            }

            // 拉取两边贡献榜前5
            String leftVipKey = RedisRtcKeys.getRtcBattleRoomInCampVipScore(battleId, LEFT_CAMP);
            Set<Tuple> leftVipSet = redisService.zReverseRangeWithScores(leftVipKey, 0, -1);
            List<Tuple> leftVipList = null;
            Tuple leftMax = null;
            if (!CollectionUtils.isEmpty(leftVipSet)) {
                leftVipList = new ArrayList<>(leftVipSet);
                leftVipList.sort(Comparator.comparingDouble(Tuple::getScore).reversed());
                leftMax = leftVipList.get(0);
            } else {
                leftVipList = Collections.emptyList();
            }

            String rightVipKey = RedisRtcKeys.getRtcBattleRoomInCampVipScore(battleId, RIGHT_CAMP);
            Set<Tuple> rightVipSet = redisService.zReverseRangeWithScores(rightVipKey, 0, -1);
            List<Tuple> rightVipList = null;
            Tuple rightMax = null;
            if (!CollectionUtils.isEmpty(rightVipSet)) {
                rightVipList = new ArrayList<>(rightVipSet);
                rightVipList.sort(Comparator.comparingDouble(Tuple::getScore).reversed());
                rightMax = rightVipList.get(0);
            } else {
                rightVipList = Collections.emptyList();
            }

            // 持久化
            List<RtcBattleRoomInVipScoreDO> vipScoreDOList = new ArrayList<>();
            if (!CollectionUtils.isEmpty(leftVipSet)) {
                for (Tuple tuple : leftVipSet) {
                    RtcBattleRoomInVipScoreDO vipScoreDO = new RtcBattleRoomInVipScoreDO();
                    vipScoreDO.setBattleId(battleId);
                    vipScoreDO.setUserId(Convert.toLong(tuple.getElement()));
                    vipScoreDO.setCamp(LEFT_CAMP);
                    vipScoreDO.setScore(Convert.toLong(tuple.getScore()));
                    vipScoreDOList.add(vipScoreDO);
                }
            }
            if (!CollectionUtils.isEmpty(rightVipSet)) {
                for (Tuple tuple : rightVipSet) {
                    RtcBattleRoomInVipScoreDO vipScoreDO = new RtcBattleRoomInVipScoreDO();
                    vipScoreDO.setBattleId(battleId);
                    vipScoreDO.setUserId(Convert.toLong(tuple.getElement()));
                    vipScoreDO.setCamp(RIGHT_CAMP);
                    vipScoreDO.setScore(Convert.toLong(tuple.getScore()));
                    vipScoreDOList.add(vipScoreDO);
                }
            }

            if (!CollectionUtils.isEmpty(vipScoreDOList)) {
                rtcBattleRoomInVipScoreDAO.batchInsert(vipScoreDOList);
            }

            // MVP 判定
            Long mvpUserId = null;
            Long mvpScore = null;
            if (battleResult == LEFT_CAMP) {
                // 取左阵营最高
                if (leftMax != null) {
                    mvpUserId = Convert.toLong(leftMax.getElement());
                    mvpScore = Convert.toLong(leftMax.getScore());
                }
            } else if (battleResult == RIGHT_CAMP) {
                // 取右阵营最高
                if (rightMax != null) {
                    mvpUserId = Convert.toLong(rightMax.getElement());
                    mvpScore = Convert.toLong(rightMax.getScore());
                }
            } else {
                // 平局 取最高
                if (leftMax != null) {
                    mvpUserId = Convert.toLong(leftMax.getElement());
                    mvpScore = Convert.toLong(leftMax.getScore());
                }

                if (rightMax != null) {
                    if (leftMax == null || rightMax.getScore() > leftMax.getScore()) {
                        mvpUserId = Convert.toLong(rightMax.getElement());
                        mvpScore = Convert.toLong(rightMax.getScore());
                    }
                }
            }

            // 移除缓存
            redisService.delKeys(Arrays.asList(leftVipKey, rightVipKey, campKey, flowKey, leftUserKey, rightUserKey, leftUserFlowKey, rightUserFlowKey));

            int leftVipSize = Math.min(leftVipList.size(), 5);
            int rightVipSize = Math.min(rightVipList.size(), 5);

            // 用户数据加载
            List<Long> userIds = new ArrayList<>();
            if (leftVipSize > 0) {
                userIds.addAll(leftVipList.subList(0, leftVipSize).stream().map(t -> Convert.toLong(t.getElement())).collect(Collectors.toList()));
            }
            if (rightVipSize > 0) {
                userIds.addAll(rightVipList.subList(0, rightVipSize).stream().map(t -> Convert.toLong(t.getElement())).collect(Collectors.toList()));
            }
            Map<Long, UserCardVO> userMap;
            if (!CollectionUtils.isEmpty(userIds)) {
                userMap = userService.getNameAndAvatarByIds(userIds).stream().collect(Collectors.toMap(UserCardVO::getUserId, Function.identity()));
            } else {
                userMap = new HashMap<>();
            }

            RtcBattleRoomInStatusMsg statusMsg = new RtcBattleRoomInStatusMsg();
            statusMsg.setBattleStatus(BattleStatusEnum.Over.getValue());
            statusMsg.setBattleResult(battleResult);

            // 拼装用户信息
            if (mvpUserId != null) {
                RtcBattleRoomInStatusMsg.BattleVip mvpUser = new RtcBattleRoomInStatusMsg.BattleVip();
                UserCardVO mvpUserCard = new UserCardVO();
                BeanUtils.copyProperties(userMap.get(mvpUserId), mvpUserCard);
                mvpUser.setUserCardVO(mvpUserCard);
                mvpUser.setScore(mvpScore);
                statusMsg.setMvpUser(mvpUser);
            }

            if (leftVipSize > 0) {
                List<RtcBattleRoomInStatusMsg.BattleVip> leftCampVipList = new ArrayList<>();
                for (int i = 0; i < leftVipSize; i++) {
                    Tuple tuple = leftVipList.get(i);
                    RtcBattleRoomInStatusMsg.BattleVip battleVip = new RtcBattleRoomInStatusMsg.BattleVip();
                    battleVip.setUserCardVO(userMap.get(Convert.toLong(tuple.getElement())));
                    battleVip.setScore(Convert.toLong(tuple.getScore()));
                    leftCampVipList.add(battleVip);
                }
                statusMsg.setLeftCampVip(leftCampVipList);
            }

            if (rightVipSize > 0) {
                List<RtcBattleRoomInStatusMsg.BattleVip> rightCampVipList = new ArrayList<>();
                for (int i = 0; i < rightVipSize; i++) {
                    Tuple tuple = rightVipList.get(i);
                    RtcBattleRoomInStatusMsg.BattleVip battleVip = new RtcBattleRoomInStatusMsg.BattleVip();
                    battleVip.setUserCardVO(userMap.get(Convert.toLong(tuple.getElement())));
                    battleVip.setScore(Convert.toLong(tuple.getScore()));
                    rightCampVipList.add(battleVip);
                }
                statusMsg.setRightCampVip(rightCampVipList);
            }

            imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, statusMsg);

            // 清除房间属性
            rtcBattleRoomPropertyProvider.setCampScoreAndCampVip(roomId, 0L, 0L, new Long[0], new Long[0]);

            return true;
        }
        return false;
    }

    private int getCamp(int seatNo) {
        if (ArrayUtil.contains(LEFT_CAMP_SEATS, seatNo)) {
            return LEFT_CAMP;
        } else if (ArrayUtil.contains(RIGHT_CAMP_SEATS, seatNo)) {
            return RIGHT_CAMP;
        } else {
            return 0;
        }
    }
}
