package com.starbuds.server.service.rtc.provider;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.lang.UUID;
import cn.hutool.crypto.SecureUtil;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.pojo.rtc.RtcRoomHeat;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.service.rtc.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 直播热度算法
 *
 * @author Lay
 * @date 2019/12/25
 */
@Component
@Slf4j
public class RtcHeatProvider {

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Resource
    RocketProducer rocketProducer;

    /**
     * 初始化热度值
     *
     * @param roomId
     */
    @Async
    public void initRoomHeatAsync(Long roomId) {
        // 重置热度值
        removeRoomHeat(roomId);

        // 设置热度键标记
        setHeatKey(roomId);
    }

    /**
     * 增加房间热度-来源：礼物
     *
     * @param roomId
     */
    public boolean addRoomHeatByGift(Long roomId, long heatValue, long duration) {
        try {
            if (duration > 0) {
                // 写入 redis
                boolean set = false;
                while (!set) {
                    set = redisService.setIfAbsent(RedisRtcKeys.getRtcRoomHeatDelay(roomId, heatValue,
                            SecureUtil.md5(UUID.fastUUID().toString(true))), "",
                            duration, TimeUnit.MILLISECONDS);
                }
            }
            return true;
        } catch (Exception e) {
            log.error("礼物增加房间热度失败", e);
        }
        return false;
    }

    /**
     * 移除热度
     *
     * @param roomId
     */
    public boolean removeRoomHeat(Long roomId) {
        try {
            // 移除热度榜
            redisService.zRemove(RedisRtcKeys.RTC_ROOM_HEAT, String.valueOf(roomId));

            // 移除最高热度
            redisService.hashDel(RedisRtcKeys.RTC_HEAT_MAX, String.valueOf(roomId));

            return true;
        } catch (Exception e) {
            log.error("热度移除失败", e);
        }
        return false;
    }

    /**
     * 更新热度值
     *
     * @param roomId
     * @param heatValue
     * @return
     */
    public Long changeRoomHeat(Long roomId, long heatValue) {
        try {
            // 写入热度值
            Double curValue = redisService.zIncrementScore(RedisRtcKeys.RTC_ROOM_HEAT, String.valueOf(roomId), Convert.toDouble(heatValue));

            Long heatValueNew = Convert.toLong(curValue);
            if (heatValueNew < 0) {
                heatValueNew = 0L;
                removeRoomHeat(roomId);
            }

            // 判断最高热度
            if (heatValueNew > Optional.ofNullable(redisService.hashGet(RedisRtcKeys.RTC_HEAT_MAX, String.valueOf(roomId))).map(Convert::toLong).orElse(0L)) {
                redisService.hashPut(RedisRtcKeys.RTC_HEAT_MAX, String.valueOf(roomId), String.valueOf(heatValueNew));
            }

            // 同步属性
            imService.setRoomProperty(roomId, "heat", String.valueOf(heatValueNew));

            return heatValueNew;
        } catch (Exception e) {
            log.error("热度值更新失败", e);
        }
        return null;
    }

    /**
     * 替换热度值
     *
     * @param roomId
     * @param heatValue
     * @return
     */
    public boolean replaceRoomHeat(Long roomId, long heatValue) {
        try {
            Double curValue = redisService.zScore(RedisRtcKeys.RTC_ROOM_HEAT, String.valueOf(roomId));
            Long oldValue = Convert.toLong(curValue);

            if (oldValue == null || oldValue < 0) {
                oldValue = 0L;
            }

            if (oldValue != heatValue) {
                redisService.zAdd(RedisRtcKeys.RTC_ROOM_HEAT, String.valueOf(roomId), Convert.toDouble(heatValue));

                // 判断最高热度
                if (heatValue > Optional.ofNullable(redisService.hashGet(RedisRtcKeys.RTC_HEAT_MAX, String.valueOf(roomId))).map(Convert::toLong).orElse(0L)) {
                    redisService.hashPut(RedisRtcKeys.RTC_HEAT_MAX, String.valueOf(roomId), String.valueOf(heatValue));
                }

                // 同步属性
                imService.setRoomProperty(roomId, "heat", String.valueOf(heatValue));
            }

            return true;
        } catch (Exception e) {
            log.error("热度值替换失败", e);
        }
        return false;
    }

    /**
     * 获取热门列表
     *
     * @param num
     * @return
     */
    public List<RtcRoomHeat> getTopN(int start, int num) {
        Set<Tuple> tuples = redisService.zReverseRangeWithScores(RedisRtcKeys.RTC_ROOM_HEAT, start, start + num - 1);
        if (CollectionUtil.isNotEmpty(tuples)) {
            List<RtcRoomHeat> liveHeats = new ArrayList<>();
            for (Tuple tuple : tuples) {
                RtcRoomHeat liveHeat = new RtcRoomHeat();
                liveHeat.setRoomId(Convert.toLong(tuple.getElement()));
                liveHeat.setHeatValue(Convert.toLong(tuple.getScore()));
                liveHeats.add(liveHeat);
            }
            liveHeats.sort(Comparator.comparingLong(RtcRoomHeat::getHeatValue).reversed());
            return liveHeats;
        }
        return Collections.emptyList();
    }

    /**
     * 获取房间的热度
     *
     * @param roomId
     * @return score
     */
    public Long getRoomHeat(Long roomId) {
        return Optional.ofNullable(redisService.zScore(RedisRtcKeys.RTC_ROOM_HEAT, String.valueOf(roomId))).orElse(0d).longValue();
    }

    /**
     * 获取房间的热度
     *
     * @param roomIds
     * @return score
     */
    public Map<Long, Long> getRoomHeatMap(List<Long> roomIds) {
        if (CollectionUtil.isEmpty(roomIds)) {
            return Collections.EMPTY_MAP;
        }
        Map<Long, Long> heatMap = new HashMap<>();
        for (Long roomId : roomIds) {
            heatMap.put(roomId, Optional.ofNullable(redisService.zScore(RedisRtcKeys.RTC_ROOM_HEAT, String.valueOf(roomId))).orElse(0d).longValue());
        }
        return heatMap;
    }

    /**
     * 获取峰值热度
     *
     * @param roomId
     * @return
     */
    public Long getRoomHeatMax(Long roomId) {
        return Optional.ofNullable(redisService.hashGet(RedisRtcKeys.RTC_HEAT_MAX, String.valueOf(roomId))).map(Convert::toLong).orElse(0L);
    }

    private void setHeatKey(Long roomId) {
        redisService.hashPut(RedisRtcKeys.RTC_HEAT_KEY, String.valueOf(roomId), String.valueOf(System.currentTimeMillis()));
    }

    /**
     * 获取热度键
     *
     * @param roomId
     * @return
     */
    public String getHeatKey(Long roomId) {
        Object obj = redisService.hashGet(RedisRtcKeys.RTC_HEAT_KEY, String.valueOf(roomId));
        if (obj != null) {
            return obj.toString();
        }
        return null;
    }

    /**
     * 移除热度键
     *
     * @param roomId
     */
    private void removeHeatKey(Long roomId) {
        redisService.hashDel(RedisRtcKeys.RTC_HEAT_KEY, String.valueOf(roomId));
    }
}
