package com.starbuds.server.service.rtc.provider;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.pojo.db.RtcRoomDO;
import com.starbuds.server.common.pojo.rtc.RtcRoomHeat;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.service.rtc.dao.RtcRoomDAO;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 热度值服务 V2
 *
 * @author Lay
 * @date 2021/7/5
 */
@Component
public class RtcHeatV2Provider {

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Resource
    RtcRoomDAO rtcRoomDAO;

    /**
     * 获取指定房间实时热度
     *
     * @param roomId
     * @return
     */
    public long getHeat(Long roomId) {
        long heat = Optional.ofNullable(redisService.zScore(RedisRtcKeys.RTC_ROOM_HEAT, String.valueOf(roomId))).map(Convert::toLong).orElse(0L);
        return Math.max(0, heat);
    }

    /**
     * 获取房间的热度
     *
     * @param roomIds
     * @return score
     */
    public Map<Long, Long> getRoomHeatMap(List<Long> roomIds) {
        if (CollectionUtil.isEmpty(roomIds)) {
            return Collections.emptyMap();
        }

        List<RtcRoomDO> roomHeats = rtcRoomDAO.selectRoomHeats(roomIds);
        if (CollectionUtil.isEmpty(roomHeats)) {
            return Collections.emptyMap();
        }

        return roomHeats.stream().collect(Collectors.toMap(RtcRoomDO::getRoomId, RtcRoomDO::getRoomHeat));
    }

    /**
     * 增加实时热度
     *
     * @param roomId
     * @param heatValue
     */
    public long addHeat(Long roomId, long heatValue) {
        if (roomId == null) {
            return 0;
        }

        long heatNew = Optional.ofNullable(redisService.zIncrementScore(RedisRtcKeys.RTC_ROOM_HEAT, String.valueOf(roomId), heatValue)).map(Convert::toLong).orElse(0L);
        syncRoomHeat(roomId, heatNew);
        return heatNew;
    }

    /**
     * 获取热门列表
     *
     * @param num
     * @return
     */
    public List<RtcRoomHeat> getTopN(int start, int num) {
        Set<Tuple> tuples = redisService.zReverseRangeWithScores(RedisRtcKeys.RTC_ROOM_HEAT, start, start + num - 1);
        if (CollectionUtil.isNotEmpty(tuples)) {
            List<RtcRoomHeat> liveHeats = new ArrayList<>();
            for (Tuple tuple : tuples) {
                RtcRoomHeat liveHeat = new RtcRoomHeat();
                liveHeat.setRoomId(Convert.toLong(tuple.getElement()));
                liveHeat.setHeatValue(Convert.toLong(tuple.getScore()));
                liveHeats.add(liveHeat);
            }
            liveHeats.sort(Comparator.comparingLong(RtcRoomHeat::getHeatValue).reversed());
            return liveHeats;
        }
        return Collections.emptyList();
    }

    /**
     * 添加热度项
     *
     * @param roomId
     * @param heatValue
     * @param expireTime
     */
    public boolean addHeatItem(Long roomId, long heatValue, long expireTime) {
        if (roomId == null) {
            return false;
        }
        redisService.hashIncreaseLong(RedisRtcKeys.getRtcRoomHeatItem(roomId), String.valueOf(getNearMinutesTime(expireTime)), heatValue);
        addHeat(roomId, heatValue);
        return true;
    }

    /**
     * 设置热度
     *
     * @param roomId
     * @param heatValue
     */
    public void setHeat(Long roomId, long heatValue) {
        if (roomId == null) {
            return;
        }

        redisService.zAdd(RedisRtcKeys.RTC_ROOM_HEAT, String.valueOf(roomId), heatValue);
    }

    public void syncRoomHeat(Long roomId, long heatValue) {
        imService.setRoomProperty(roomId, "heat", String.valueOf(heatValue));
    }

    /**
     * 获取时间戳最近的分钟数
     * 为了减短队列长度，将时间戳统一归置为分钟最小单位
     * 如果有多余秒数则分钟数加一
     * 并将秒位、毫秒位清零
     *
     * @param time
     * @return
     */
    private long getNearMinutesTime(long time) {
        // 获取最靠近的那一分钟数
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(time);
        if (calendar.get(Calendar.SECOND) > 0) {
            calendar.add(Calendar.MINUTE, 1);
        }
        // 秒、毫秒清零
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTimeInMillis();
    }
}
