package com.starbuds.server.service.rtc.provider;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.pojo.rtc.RtcScoreAssister;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.user.UserService;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 相亲模式心动值
 *
 * @author Lay
 * @date 2020/8/5
 */
@Component
public class RtcLoveScoreProvider {

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Resource
    RtcLoveScorePersistProvider rtcLoveScorePersistProvider;

    /**
     * 初始化
     *
     * @param roomId
     */
    public void init(Long roomId) {
        // 移除心动值
        redisService.delKey(RedisRtcKeys.getRtcLoveScore(roomId));

        List<String> keys = redisService.getKeys(RedisRtcKeys.getRtcLoveAssistsPattern(roomId));
        if (CollectionUtil.isNotEmpty(keys)) {
            Long beginTime = Optional.ofNullable(redisService.get(RedisRtcKeys.getRtcLoveTime(roomId))).map(Convert::toLong).orElse(0L);

            if (beginTime > 0) {
                // 读取数据持久化到轮次流水
                Map<Long, Set<Tuple>> userAssistsMap = new HashMap<>();
                for (String key : keys) {
                    Set<Tuple> tupleSet = redisService.zRangeWithScores(key, 0, -1);
                    if (tupleSet != null) {
                        Long userId = Convert.toLong(key.split("_")[1]);
                        userAssistsMap.put(userId, tupleSet);
                    }
                }

                // 异步持久化
                rtcLoveScorePersistProvider.persistAsync(roomId, beginTime, userAssistsMap);
            }

            redisService.delKeys(keys);
        }

        // 重置轮次时间
        redisService.set(RedisRtcKeys.getRtcLoveTime(roomId), String.valueOf(System.currentTimeMillis()));
    }

    /**
     * 增加心动值
     *
     * @param roomId
     * @param userId
     * @param scoreValue
     * @return
     */
    public Long addScore(Long roomId, Long userId, Long scoreValue, Long assistUserId) {
        Long newScore = redisService.hashIncreaseLong(RedisRtcKeys.getRtcLoveScore(roomId), String.valueOf(userId), scoreValue);

        // 增加贡献榜
        redisService.zIncrementScore(RedisRtcKeys.getRtcLoveAssists(roomId, userId), String.valueOf(assistUserId), scoreValue);

        return newScore;
    }

    /**
     * 获取心动值助攻用户列表
     *
     * @param roomId
     * @param userId
     * @return
     */
    public List<RtcScoreAssister> getAssisterList(Long roomId, Long userId) {
        Set<Tuple> tupleSet = redisService.zReverseRangeWithScores(RedisRtcKeys.getRtcLoveAssists(roomId, userId), 0, -1);
        if (CollectionUtil.isNotEmpty(tupleSet)) {
            // 取出所有用户 id
            List<Long> userIds = tupleSet.stream().map(t -> Convert.toLong(t.getElement())).collect(Collectors.toList());
            Map<Long, UserCardVO> userMap = userService.getNameAndAvatarByIds(userIds).stream().collect(Collectors.toMap(UserCardVO::getUserId, Function.identity()));

            List<RtcScoreAssister> assisterList = new ArrayList<>();
            for (Tuple tuple : tupleSet) {
                RtcScoreAssister assister = new RtcScoreAssister();
                UserCardVO userCard = userMap.get(Convert.toLong(tuple.getElement()));
                assister.setUserId(userCard.getUserId());
                assister.setUserName(userCard.getUserName());
                assister.setUserAvatar(userCard.getUserAvatar());
                assister.setUserSex(userCard.getUserSex());
                assister.setAssistScore(Convert.toLong(tuple.getScore()));
                assisterList.add(assister);
            }

            // 二次排序
            assisterList.sort(Comparator.comparingLong(RtcScoreAssister::getAssistScore).reversed());
            return assisterList;
        }
        return Collections.emptyList();
    }

    /**
     * 获取心动值
     *
     * @param roomId
     * @param userId
     * @return
     */
    public Long getScore(Long roomId, Long userId) {
        return Optional.ofNullable(redisService.hashGet(RedisRtcKeys.getRtcLoveScore(roomId), String.valueOf(userId))).map(Convert::toLong).orElse(0L);
    }

    public void clearScore(Long roomId, Long userId) {
        redisService.hashDel(RedisRtcKeys.getRtcLoveScore(roomId), String.valueOf(userId));
        redisService.delKey(RedisRtcKeys.getRtcLoveAssists(roomId, userId));
    }
}
