package com.starbuds.server.service.rtc.provider;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.user.SexEnum;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.pojo.data.MedalShinyStatus;
import com.starbuds.server.common.pojo.message.im.rtc.RtcQueueClearMsg;
import com.starbuds.server.common.pojo.message.im.rtc.RtcQueueJoinMsg;
import com.starbuds.server.common.pojo.message.im.rtc.RtcQueueLeaveMsg;
import com.starbuds.server.common.pojo.rtc.RtcQueueUser;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.UserMedalService;
import com.starbuds.server.common.service.user.UserService;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 队列服务
 *
 * @author Lay
 * @date 2020/7/7
 */
@Component
public class RtcQueueProvider {

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    private UserMedalService userMedalService;

    /**
     * 加入队列
     *
     * @param roomId
     * @param userId
     * @return
     */
    public boolean addUser(Long roomId, Long userId) {
        if (isExists(roomId, userId)) {
            return false;
        }

        // 积分为加入的时间戳
        double curTime = System.currentTimeMillis() / 1000d;
        boolean success = redisService.zAdd(RedisRtcKeys.getRtcRoomQueue(roomId), String.valueOf(userId), curTime);
        if (success) {
            // 加入成功
            RtcQueueJoinMsg queueJoinMsg = new RtcQueueJoinMsg();
            queueJoinMsg.setRoomId(roomId);
            queueJoinMsg.setUserId(userId);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, userId, null, roomId, queueJoinMsg);
        }
        return success;
    }

    /**
     * 移出队列
     *
     * @param roomId
     * @param userId
     * @return
     */
    public boolean removeUser(Long roomId, Long userId) {
        boolean success = redisService.zRemove(RedisRtcKeys.getRtcRoomQueue(roomId), String.valueOf(userId)) > 0;
        if (success) {
            // 移出成功
            RtcQueueLeaveMsg queueLeaveMsg = new RtcQueueLeaveMsg();
            queueLeaveMsg.setRoomId(roomId);
            queueLeaveMsg.setUserId(userId);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, userId, null, roomId, queueLeaveMsg);
        }
        return success;
    }

    /**
     * 清空队列
     *
     * @param roomId
     * @return
     */
    public boolean clear(Long roomId) {
        boolean success = redisService.delKey(RedisRtcKeys.getRtcRoomQueue(roomId));
        if (success) {
            // 清除成功
            RtcQueueClearMsg queueClearMsg = new RtcQueueClearMsg();
            queueClearMsg.setRoomId(roomId);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, queueClearMsg);
        }
        return success;
    }

    /**
     * 是否存在
     *
     * @param roomId
     * @param userId
     * @return
     */
    public boolean isExists(Long roomId, Long userId) {
        return redisService.zScore(RedisRtcKeys.getRtcRoomQueue(roomId), String.valueOf(userId)) != null;
    }

    /**
     * 统计队列人数
     *
     * @param roomId
     * @return
     */
    public int count(Long roomId) {
        return Optional.ofNullable(redisService.zCard(RedisRtcKeys.getRtcRoomQueue(roomId))).map(Convert::toInt).orElse(0);
    }

    /**
     * 获取队列用户
     *
     * @param roomId
     * @return
     */
    public List<RtcQueueUser> getUsers(Long roomId) {
        Set<Tuple> tupleSet = redisService.zRangeWithScores(RedisRtcKeys.getRtcRoomQueue(roomId), 0, -1);
        if (CollectionUtil.isNotEmpty(tupleSet)) {
            List<Long> userIds = new ArrayList<>();
            Map<Long, Long> userSitTimeMap = new HashMap<>();
            for (Tuple tuple : tupleSet) {
                Long userId = Convert.toLong(tuple.getElement());
                userIds.add(userId);
                userSitTimeMap.put(userId, Convert.toLong(tuple.getScore() * 1000));
            }

            // 查询用户 填充
            List<RtcQueueUser> list = new ArrayList<>();
            List<UserCardVO> users = userService.getNameAndAvatarByIds(userIds);
            if (CollectionUtil.isNotEmpty(users)) {
                for (UserCardVO userCard : users) {
                    RtcQueueUser queueUser = new RtcQueueUser();
                    BeanUtils.copyProperties(userCard, queueUser);
                    queueUser.setUserSex(Optional.ofNullable(userCard.getUserSex()).orElse(SexEnum.MALE.getCode()));
                    Long userId = userCard.getUserId();
                    queueUser.setSitTime(userSitTimeMap.get(userId));

                    MedalShinyStatus medalShinyStatus = userMedalService.getMedalShinyStatus(userId);
                    queueUser.setMedalShinyStatus(medalShinyStatus);
                    list.add(queueUser);
                }

                // 根据加入时间排序
                list.sort(Comparator.comparingLong(RtcQueueUser::getSitTime));
                return list;
            }
        }
        return Collections.emptyList();
    }

    /**
     * 获取队列用户 id 列表
     *
     * @param roomId
     * @return
     */
    public List<Long> getUserIds(Long roomId) {
        Set<Tuple> tupleSet = redisService.zRangeWithScores(RedisRtcKeys.getRtcRoomQueue(roomId), 0, -1);
        if (CollectionUtil.isNotEmpty(tupleSet)) {
            List<Tuple> list = new ArrayList<>(tupleSet);
            list.sort(Comparator.comparingDouble(Tuple::getScore));
            return list.stream().map(t -> Convert.toLong(t.getElement())).collect(Collectors.toList());
        }
        return Collections.emptyList();
    }
}
