package com.starbuds.server.service.rtc.provider;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.user.SexEnum;
import com.starbuds.server.common.pojo.core.Tuple;
import com.starbuds.server.common.pojo.data.HeadwearVO;
import com.starbuds.server.common.pojo.media.LiveRankItem;
import com.starbuds.server.common.pojo.rtc.RtcRankItem;
import com.starbuds.server.common.pojo.user.UserCardVO;
import com.starbuds.server.common.service.data.HeadwearService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.service.user.UserService;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.rtc.dao.StatRtcRoomDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.starbuds.server.service.rtc.util.SpringContextUtil.isDev;

/**
 * 排行服务
 *
 * @author Lay
 * @date 2020/7/14
 */
@Slf4j
@Component
public class RtcRankProvider {

    @Resource
    StatRtcRoomDAO statRtcRoomDAO;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Reference(version = "1.0.0")
    HeadwearService headwearService;

    static {
        StringBuilder sb = new StringBuilder();
        // 加日榜
        sb.append("redis.call('ZINCRBY', KEYS[1], ARGV[2], ARGV[1]);");
        // 周榜
        sb.append("redis.call('ZINCRBY', KEYS[2], ARGV[2], ARGV[1]);");
        // 月榜
        sb.append("redis.call('ZINCRBY', KEYS[3], ARGV[2], ARGV[1]);");
        LUA_INCR = sb.toString();
    }

    static {
        StringBuilder sb = new StringBuilder();
        sb.append("local t = redis.call('TIME');");
        sb.append("return t[1]");
        LUA_TIME = sb.toString();
    }

    /**
     * 中央时间戳
     */
    public static final String LUA_TIME;

    /**
     * 加榜
     */
    public static final String LUA_INCR;

    /**
     * 获取 Redis 时间戳
     *
     * @return
     */
    private long getRedisTime() {
        return redisService.luaLong(LUA_TIME, new ArrayList<>(), new ArrayList<>());
    }

    public String[] getRankTimes() {
        long redisTime = getRedisTime();
        String[] times = new String[3];

        // 日
        Date date = new Date(redisTime * 1000);
        SimpleDateFormat sdfDay = new SimpleDateFormat("yyyyMMdd");
        times[0] = sdfDay.format(date);

        // 基础周为 2020年1月6日第一周
        long baseTime = 1578240000;
        long secDiff = redisTime - baseTime;
        double dayDiff = secDiff / 3600 / 24;
        int week = (int) (dayDiff / 7 + 1);
        times[1] = String.valueOf(week);

        // 月
        SimpleDateFormat sdfMonth = new SimpleDateFormat("yyyyMM");
        times[2] = sdfMonth.format(date);

        return times;
    }

    /**
     * 获取上期榜单
     *
     * @return
     */
    public String[] getPreviousRankTimes() {

        String[] previousArray = new String[3];

        String[] rankTimes = getRankTimes();
        // 昨日
        SimpleDateFormat sdfDay = new SimpleDateFormat("yyyyMMdd");
        Calendar curCal = Calendar.getInstance();
        curCal.add(Calendar.DAY_OF_YEAR, -1);
        previousArray[0] = Convert.toStr(sdfDay.format(curCal.getTime()));
        // 上周
        previousArray[1] = Convert.toStr(Convert.toInt(rankTimes[1]) - 1);
        // 上月
        SimpleDateFormat sdfMonth = new SimpleDateFormat("yyyyMM");
        Calendar monthCal = Calendar.getInstance();
        monthCal.add(Calendar.MONTH, -1);
        previousArray[2] = Convert.toStr(sdfMonth.format(curCal.getTime()));
        return previousArray;
    }

    public List<LiveRankItem> getRankItems(String key, long start, long end) {
        Set<Tuple> rankData = redisService.zReverseRangeWithScores(key, start, end);

        if (CollectionUtil.isNotEmpty(rankData)) {
            List<LiveRankItem> liveRankItems = new ArrayList<>();

            // 需要加载的用户
            Set<Long> userIds = new HashSet<>();

            for (Tuple data : rankData) {
                Long userId = Convert.toLong(data.getElement());
                userIds.add(userId);
                LiveRankItem liveRankItem = new LiveRankItem();
                liveRankItem.setUserId(userId);
                liveRankItem.setAmount(Convert.toLong(data.getScore()));
                liveRankItems.add(liveRankItem);
            }

            // 拉取用户数据
            List<UserCardVO> userCardVOList = userService.getNameAndAvatarByIds(new ArrayList<>(userIds));
            Map<Long, UserCardVO> userMap = userCardVOList.stream().collect(Collectors.toMap(UserCardVO::getUserId, Function.identity()));

            UserCardVO defaultUser = new UserCardVO();
            defaultUser.setUserName("已注销");
            defaultUser.setUserSex(SexEnum.FEMALE.getCode());
            defaultUser.setIsOnline(CommonStatus.Disabled);
            // 排序
            liveRankItems.sort(Comparator.comparingLong(LiveRankItem::getAmount).reversed());
            int index = 0;
            // 房间
            Map<Long, Long> userRoomMap = rtcUserService.getUserRoomIdMap(userIds);
            // 头饰
            Map<Long, HeadwearVO> headwearMap = headwearService.getUsersEnabledHeadwear(new ArrayList<>(userIds));
            for (LiveRankItem liveRankItem : liveRankItems) {
                index++;
                UserCardVO userCardVO = userMap.getOrDefault(liveRankItem.getUserId(), defaultUser);
                if (userMap.containsKey(liveRankItem.getUserId())) {
                    liveRankItem.setUserName(userCardVO.getUserName());
                    liveRankItem.setUserAvatar(userCardVO.getUserAvatar());
                    liveRankItem.setWealthLevel(userCardVO.getWealthLevel());
                    liveRankItem.setIsOnline(userCardVO.getIsOnline());
                    liveRankItem.setUserSex(userCardVO.getUserSex());
                }
                liveRankItem.setRoomId(CollectionUtil.isNotEmpty(userRoomMap) ? userRoomMap.get(liveRankItem.getUserId()) : null);
                if (CollectionUtil.isNotEmpty(headwearMap)) {
                    HeadwearVO headwearVO = headwearMap.get(liveRankItem.getUserId());
                    liveRankItem.setAnimationFile(headwearVO != null ? headwearVO.getAnimationFile() : null);
                }

                liveRankItem.setRankNo(index);
            }

            return liveRankItems;
        }

        return Collections.emptyList();
    }

    public void addDiamondRankAmount(Long roomId, Long userId, Long amount) {
        if (amount < 1L) {
            return;
        }

        String[] times = getRankTimes();

        // 加榜
        List<String> keys = new ArrayList<>();
        keys.add(RedisRtcKeys.getRtcRankDiamondDay(roomId, times[0]));
        keys.add(RedisRtcKeys.getRtcRankDiamondWeek(roomId, times[1]));
        keys.add(RedisRtcKeys.getRtcRankDiamondMonth(roomId, times[2]));

        // 参数
        List<String> args = new ArrayList<>();
        args.add(String.valueOf(userId));
        args.add(String.valueOf(amount));

        // 加榜
        redisService.luaLong(LUA_INCR, keys, args);

        if (roomId != 0) {
            addDiamondRankAmount(0L, userId, amount);
        }
    }

    public void addDustRankAmount(Long roomId, Long userId, Long amount) {
        if (amount < 1L) {
            return;
        }

        String[] times = getRankTimes();
        if (isDev()) {
            log.info("times = {}", LogUtil.obj2JsonObj(times));
        }

        // 加榜
        List<String> keys = new ArrayList<>();
        keys.add(RedisRtcKeys.getRtcRankDustDay(roomId, times[0]));
        keys.add(RedisRtcKeys.getRtcRankDustWeek(roomId, times[1]));
        keys.add(RedisRtcKeys.getRtcRankDustMonth(roomId, times[2]));

        // 参数
        List<String> args = new ArrayList<>();
        args.add(String.valueOf(userId));
        args.add(String.valueOf(amount));

        // 加榜
        redisService.luaLong(LUA_INCR, keys, args);

        if (roomId != 0) {
            addDustRankAmount(0L, userId, amount);
        }
    }

    /**
     * 获取小时榜
     *
     * @param statTime
     * @return
     */
    public List<RtcRankItem> getRtcHourRank(Long statTime) {
        List<RtcRankItem> rankItems = statRtcRoomDAO.selectRtcHourRank(statTime);
        List<RtcRankItem> list = rankItems.stream().filter(e -> e.getNeedPass() == CommonStatus.Disabled).collect(Collectors.toList());
        List<RtcRankItem> needPassList = rankItems.stream().filter(e -> e.getNeedPass() == CommonStatus.Normal).collect(Collectors.toList());
        list.addAll(needPassList);
        return list;
    }
}
