package com.starbuds.server.service.rtc.provider;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.user.SexEnum;
import com.starbuds.server.common.pojo.message.im.rtc.RtcLoveHatMsg;
import com.starbuds.server.common.pojo.message.im.rtc.RtcScoreChangedMsg;
import com.starbuds.server.common.pojo.rtc.RtcUserScore;
import com.starbuds.server.common.pojo.user.UserDTO;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 心动值服务
 *
 * @author Lay
 * @date 2020/7/11
 */
@Slf4j
@Component
public class RtcScoreProvider {

    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    UserService userService;

    public void init(Long roomId) {
        String rtcUserScore = RedisRtcKeys.getRtcUserScore(roomId);
        String rtcScoreTime = RedisRtcKeys.getRtcScoreTime(roomId);
        redisService.delKey(rtcUserScore);
        redisService.delKey(rtcScoreTime);
    }

    /**
     * 增加心动值
     *
     * @param roomId
     * @param seatNo
     * @param value
     * @return
     */
    public Long addScore(Long roomId, int seatNo, Long value) {
        Long newScore = redisService.hashIncreaseLong(RedisRtcKeys.getRtcSeatScore(roomId), String.valueOf(seatNo), value);

        // 发送消息
        RtcScoreChangedMsg scoreChangedMsg = new RtcScoreChangedMsg();
        scoreChangedMsg.setRoomId(roomId);
        scoreChangedMsg.setSeatNo(seatNo);
        scoreChangedMsg.setScoreValue(newScore);
        imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, scoreChangedMsg);

        return newScore;
    }

    /**
     * 增加心动值
     *
     * @param roomId
     * @param userId
     * @param value
     * @return
     */
    public Long addScore(Long roomId, Long userId, Long value) {
        Long newScore = redisService.hashIncreaseLong(RedisRtcKeys.getRtcUserScore(roomId), Convert.toStr(userId), value);

        // 发送消息
        RtcScoreChangedMsg scoreChangedMsg = new RtcScoreChangedMsg();
        scoreChangedMsg.setRoomId(roomId);
        scoreChangedMsg.setUserId(userId);
        scoreChangedMsg.setScoreValue(newScore);
        imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, scoreChangedMsg);

        return newScore;
    }

    /**
     * 清空某位置的心动值
     *
     * @param roomId
     * @param seatNo
     */
    public void clearScore(Long roomId, int seatNo) {
        redisService.hashPut(RedisRtcKeys.getRtcSeatScore(roomId), String.valueOf(seatNo), "0");
        // 重置时间戳
        redisService.hashPut(RedisRtcKeys.getRtcScoreTime(roomId), String.valueOf(seatNo), String.valueOf(System.currentTimeMillis()));

        RtcScoreChangedMsg scoreChangedMsg = new RtcScoreChangedMsg();
        scoreChangedMsg.setRoomId(roomId);
        scoreChangedMsg.setSeatNo(seatNo);
        scoreChangedMsg.setScoreValue(0L);
        imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, scoreChangedMsg);
    }

    /**
     * 清空指定用户心动值
     *
     * @param roomId
     * @param userId
     */
    public void clearScore(Long roomId, Long userId) {
        redisService.hashDel(RedisRtcKeys.getRtcUserScore(roomId), Convert.toStr(userId));
        // 重置时间戳
        redisService.hashDel(RedisRtcKeys.getRtcScoreTime(roomId), Convert.toStr(userId));
        RtcScoreChangedMsg scoreChangedMsg = new RtcScoreChangedMsg();
        scoreChangedMsg.setRoomId(roomId);
        scoreChangedMsg.setUserId(userId);
        scoreChangedMsg.setScoreValue(0L);
        imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, scoreChangedMsg);
    }

    /**
     * 清空所有心动值
     *
     * @param roomId
     */
    public void clearAll(Long roomId) {
        init(roomId);
        RtcScoreChangedMsg scoreChangedMsg = new RtcScoreChangedMsg();
        scoreChangedMsg.setRoomId(roomId);
        scoreChangedMsg.setUserId(-1L);
        scoreChangedMsg.setScoreValue(0L);
        imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, scoreChangedMsg);
    }

    /**
     * 获取心动值
     *
     * @param roomId
     * @return
     */
    public Map<Integer, Long> getScoreMap(Long roomId) {
        Map<Object, Object> tempMap = redisService.hashGetAll(RedisRtcKeys.getRtcSeatScore(roomId));
        if (tempMap != null) {
            Map<Integer, Long> scoreMap = new HashMap<>();
            for (Map.Entry<Object, Object> entry : tempMap.entrySet()) {
                scoreMap.put(Convert.toInt(entry.getKey()), Convert.toLong(entry.getValue()));
            }
            return scoreMap;
        }
        return null;
    }

    /**
     * 获取心动值
     *
     * @param roomId
     * @return
     */
    public Map<Long, Long> getUserScoreMap(Long roomId) {
        Map<Object, Object> tempMap = redisService.hashGetAll(RedisRtcKeys.getRtcUserScore(roomId));
        if (CollectionUtil.isNotEmpty(tempMap)) {
            // userId => score
            Map<Long, Long> scoreMap = new HashMap<>();
            for (Map.Entry<Object, Object> entry : tempMap.entrySet()) {
                scoreMap.put(Convert.toLong(entry.getKey()), Convert.toLong(entry.getValue()));
            }
            return scoreMap;
        }
        return Collections.EMPTY_MAP;
    }

    /**
     * 心动值时间
     *
     * @param roomId
     * @return
     */
    public Map<Long, Long> getScoreTimeMap(Long roomId) {
        Map<Object, Object> tempMap = redisService.hashGetAll(RedisRtcKeys.getRtcScoreTime(roomId));
        if (CollectionUtil.isNotEmpty(tempMap)) {
            // userId => timestamp
            Map<Long, Long> scoreMap = new HashMap<>();
            for (Map.Entry<Object, Object> entry : tempMap.entrySet()) {
                scoreMap.put(Convert.toLong(entry.getKey()), Convert.toLong(entry.getValue()));
            }
            return scoreMap;
        }
        return Collections.EMPTY_MAP;
    }

    /**
     * 获取用户心动值列表
     *
     * @param roomId
     * @return
     */
    public List<RtcUserScore> getUserScoreList(Long roomId) {
        Map<Long, Long> userScoreMap = getUserScoreMap(roomId);
        if (CollectionUtil.isNotEmpty(userScoreMap)) {
            Set<Long> userIdSet = userScoreMap.keySet();
            List<UserDTO> users = userService.getUsers(new ArrayList<>(userIdSet));
            Map<Long, String> userNameMap = users.stream().collect(Collectors.toMap(UserDTO::getUserId, UserDTO::getUserName));
            Map<Long, String> userAvatarMap = users.stream().collect(Collectors.toMap(UserDTO::getUserId, e -> Optional.ofNullable(e.getUserAvatar()).orElse("")));
            Map<Long, Integer> userSexMap = users.stream().collect(Collectors.toMap(UserDTO::getUserId, t -> t.getUserSex() == null ? SexEnum.FEMALE.getCode() : t.getUserSex()));
            List<RtcUserScore> list = new ArrayList<>();
            Map<Long, Long> timeMap = getScoreTimeMap(roomId);
            for (Map.Entry<Long, Long> entry : userScoreMap.entrySet()) {
                RtcUserScore userScore = new RtcUserScore();
                userScore.setUserId(entry.getKey());
                userScore.setScoreValue(entry.getValue());
                userScore.setTimestamp(timeMap.get(entry.getKey()));
                userScore.setUserName(CollectionUtil.isNotEmpty(userNameMap) ? userNameMap.get(entry.getKey()) : "");
                userScore.setUserAvatar(CollectionUtil.isNotEmpty(userAvatarMap) ? userAvatarMap.get(entry.getKey()) : "");
                userScore.setUserSex(userSexMap.get(entry.getKey()));
                list.add(userScore);
            }
            // 降序排列
            return list.stream().sorted(Comparator.comparing(RtcUserScore::getScoreValue).reversed()).collect(Collectors.toList());
        }
        return Collections.emptyList();
    }

    private static final List<Integer> redSeats = Arrays.asList(1, 2, 5, 6);
    private static final List<Integer> blueSeats = Arrays.asList(3, 4, 7, 8);

    /**
     * 帽子计算
     *
     * @param roomId
     */
    public void refreshHat(Long roomId) {
        Map<Integer, Long> scoreMap = getScoreMap(roomId);
        if (CollectionUtil.isNotEmpty(scoreMap)) {
            long redScoreMax = 0L;
            int redSeatNo = -1;
            long blueScoreMax = 0L;
            int blueSeatNo = -1;
            for (Map.Entry<Integer, Long> entry : scoreMap.entrySet()) {
                if (redSeats.contains(entry.getKey())) {
                    if (entry.getValue() > redScoreMax) {
                        redScoreMax = entry.getValue();
                        redSeatNo = entry.getKey();
                    }
                } else if (blueSeats.contains(entry.getKey())) {
                    if (entry.getValue() > blueScoreMax) {
                        blueScoreMax = entry.getValue();
                        blueSeatNo = entry.getKey();
                    }
                }
            }

            if (redSeatNo > -1) {
                // 判断级别
                int hatLevel = getHatLevel(redScoreMax);
                if (hatLevel > 0) {
                    RtcLoveHatMsg loveHatMsg = new RtcLoveHatMsg();
                    loveHatMsg.setHatLevel(hatLevel);
                    loveHatMsg.setRoomId(roomId);
                    loveHatMsg.setSeatNo(redSeatNo);
                    imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, loveHatMsg);
                }
            }

            if (blueSeatNo > -1) {
                // 判断级别
                int hatLevel = getHatLevel(blueScoreMax);
                if (hatLevel > 0) {
                    RtcLoveHatMsg loveHatMsg = new RtcLoveHatMsg();
                    loveHatMsg.setHatLevel(hatLevel);
                    loveHatMsg.setRoomId(roomId);
                    loveHatMsg.setSeatNo(blueSeatNo);
                    imService.sendRtcRoomMsg(ImTargetType.GROUP, null, null, roomId, loveHatMsg);
                }
            }
        }
    }

    private int getHatLevel(long scoreValue) {
        if (scoreValue >= 52000) {
            return 3;
        } else if (scoreValue >= 15000) {
            return 2;
        } else if (scoreValue >= 5000) {
            return 1;
        }
        return 0;
    }
}
