package com.starbuds.server.service.rtc.provider;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyuncs.CommonRequest;
import com.aliyuncs.CommonResponse;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.IAcsClient;
import com.aliyuncs.http.MethodType;
import com.aliyuncs.profile.DefaultProfile;
import com.starbuds.server.common.constant.cache.RedisRtcKeys;
import com.starbuds.server.common.constant.common.TimeConstants;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.rtc.RoomTypeEnum;
import com.starbuds.server.common.constant.rtc.RtcBusinessTypeEnum;
import com.starbuds.server.common.constant.statistics.StatRtcRoomTypeEnum;
import com.starbuds.server.common.constant.statistics.StatTimeType;
import com.starbuds.server.common.constant.statistics.StatUserTypeEnum;
import com.starbuds.server.common.pojo.db.RtcRoomSeatLogDO;
import com.starbuds.server.common.pojo.message.AliyunConfig;
import com.starbuds.server.common.pojo.message.im.rtc.*;
import com.starbuds.server.common.pojo.mq.QueueRtcCheckStatusMsg;
import com.starbuds.server.common.pojo.mq.QueueRtcSitUpSeatMsg;
import com.starbuds.server.common.service.core.ConfigService;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.media.VoiceChatService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.statistics.StatRtcRoomService;
import com.starbuds.server.common.service.statistics.StatUserService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.rtc.dao.RtcRoomDAO;
import com.starbuds.server.service.rtc.dao.RtcRoomRoleDAO;
import com.starbuds.server.service.rtc.dao.RtcRoomSeatLogDAO;
import com.starbuds.server.service.rtc.rocket.RocketProducer;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 麦位服务
 *
 * @author Lay
 * @date 2020/7/7
 */
@Component
@Slf4j
public class RtcSeatProvider {
    @Reference(version = "1.0.0")
    RedisService redisService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    ConfigService configService;

    @Reference(version = "1.0.0")
    VoiceChatService voiceChatService;

    @Reference(version = "1.0.0")
    private StatUserService statUserService;

    @Reference(version = "1.0.0")
    private StatRtcRoomService statRtcRoomService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Resource
    RtcQueueProvider rtcQueueProvider;

    @Resource
    RtcScoreProvider rtcScoreProvider;

    @Resource
    RtcLoveScoreProvider rtcLoveScoreProvider;

    @Resource
    RocketProducer rocketProducer;

    @Resource
    RtcRoomRoleDAO rtcRoomRoleDAO;

    @Resource
    RtcRoomSeatLogDAO rtcRoomSeatLogDAO;

    @Resource
    RtcRoomDAO rtcRoomDAO;

    @Value("${aliyun.rtc.appId}")
    String aliyunRtcAppId;

    /**
     * 上麦位
     *
     * @param roomId
     * @param userId
     * @param seatNo
     * @return
     */
    public int sitSeat(Long roomId, Long userId, Integer seatNo, boolean skipLock, String appVer) {
        int seat = -1;

        // 锁定用户
        String lockKey = RedisRtcKeys.getRtcUserSitLock(userId);
        if (!redisService.setIfAbsent(lockKey, String.valueOf(userId), TimeConstants.MIN_MILLS, TimeUnit.MILLISECONDS)) {
            return seat;
        }

        // 判断是否自动麦位
        if (seatNo == null) {
            // 自动麦位支持 1-8
            // 个播支持1-4
            int seatNum = 9;
            Integer roomBusinessType = rtcRoomDAO.selectRoomBusinessType(roomId);
            if (RtcBusinessTypeEnum.RTC_SINGLE.equals(RtcBusinessTypeEnum.toEnum(roomBusinessType))) {
                seatNum = 5;
            }
            for (int i = 1; i < seatNum; i++) {
                if (touchSeat(roomId, i, userId, skipLock)) {
                    seat = i;
                    break;
                }
            }
        } else if (touchSeat(roomId, seatNo, userId, skipLock)) {
            seat = seatNo;
        }

        // 移除锁
        redisService.delKey(lockKey);

        if (seat == -1) {
            return seat;
        }

        //记录上麦日志
        RtcRoomSeatLogDO logDO = new RtcRoomSeatLogDO();
        logDO.setHisId(identityService.getId());
        logDO.setRoomId(roomId);
        logDO.setSeatNo(Integer.valueOf(seat));
        logDO.setUserId(userId);
        logDO.setAction(1);
        logDO.setTimestamp(System.currentTimeMillis());
        rtcRoomSeatLogDAO.insert(logDO);

        if (skipLock && seat > 0) {
            // 被动上麦 给目标用户发送上麦消息
            RtcJoinSeatMsg joinSeatMsg = new RtcJoinSeatMsg();
            joinSeatMsg.setUserId(userId);
            joinSeatMsg.setRoomId(roomId);
            joinSeatMsg.setSeatNo(seat);
            joinSeatMsg.setAuthInfo(voiceChatService.getAuthInfo(roomId, userId, appVer));
            imService.sendRtcRoomMsg(ImTargetType.USER, null, userId, roomId, joinSeatMsg);
        }

        // 如果在麦序中，将用户从麦序中移除
        if (rtcQueueProvider.removeUser(roomId, userId)) {
            RtcQueueLeaveMsg queueLeaveMsg = new RtcQueueLeaveMsg();
            queueLeaveMsg.setRoomId(roomId);
            queueLeaveMsg.setUserId(userId);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, userId, null, roomId, queueLeaveMsg);
        }

        // 判断是否相亲房
        if (RoomTypeEnum.LOVE.getValue() == rtcRoomDAO.selectRoomType(roomId)) {
            // 设置心动值
            rtcScoreProvider.addScore(roomId, userId, rtcLoveScoreProvider.getScore(roomId, userId));
        }

        // 发送消息
        RtcSitSeatMsg sitSeatMsg = new RtcSitSeatMsg();
        sitSeatMsg.setRoomId(roomId);
        sitSeatMsg.setSeatNo(seat);
        sitSeatMsg.setUser(rtcRoomRoleDAO.selectUserById(roomId, userId));
        imService.sendRtcRoomMsg(ImTargetType.GROUP, userId, null, roomId, sitSeatMsg);

        // 发送上麦mq消息
        QueueRtcSitUpSeatMsg msg = new QueueRtcSitUpSeatMsg();
        msg.setSeatNo(seatNo);
        msg.setRoomId(roomId);
        msg.setUserId(userId);
        rocketProducer.sendMsg(RocketTopic.TPC_NORMAL, RocketTag.RTC_SIT_UP_SEAT, msg.serialize());

        return seat;
    }

    /**
     * 下麦位
     *
     * @param roomId
     * @param seatNo
     * @param isForce
     * @return
     */
    public int leaveSeat(Long roomId, int seatNo, boolean isForce, String appVer) {
        if (seatNo < 0) {
            return -1;
        }

        Long userId = getSeatUserId(roomId, seatNo);

        if (userId != null) {
            redisService.hashDel(RedisRtcKeys.getRtcRoomSeat(roomId), String.valueOf(seatNo));
            // 重置静音状态
            cancelMicSilenced(roomId, seatNo);

            if (isForce) {
                RtcExitSeatMsg exitSeatMsg = new RtcExitSeatMsg();
                exitSeatMsg.setRoomId(roomId);
                exitSeatMsg.setSeatNo(seatNo);
                exitSeatMsg.setUserId(userId);
                exitSeatMsg.setAuthInfo(voiceChatService.getAuthInfo(roomId, userId, appVer));
                imService.sendRtcRoomMsg(ImTargetType.USER, null, userId, roomId, exitSeatMsg);
            }

            // 发送消息
            RtcLeaveSeatMsg leaveSeatMsg = new RtcLeaveSeatMsg();
            leaveSeatMsg.setRoomId(roomId);
            leaveSeatMsg.setSeatNo(seatNo);
            leaveSeatMsg.setUserId(userId);
            imService.sendRtcRoomMsg(ImTargetType.GROUP, userId, null, roomId, leaveSeatMsg);

            // 判断是否相亲房
            if (RoomTypeEnum.LOVE.getValue() == rtcRoomDAO.selectRoomType(roomId)) {
                // 设置心动值
                rtcScoreProvider.clearScore(roomId, userId);
            }

            //记录下麦日志
            RtcRoomSeatLogDO logDO = new RtcRoomSeatLogDO();
            logDO.setHisId(identityService.getId());
            logDO.setRoomId(roomId);
            logDO.setSeatNo(seatNo);
            logDO.setUserId(userId);
            logDO.setAction(0);
            logDO.setTimestamp(System.currentTimeMillis());
            rtcRoomSeatLogDAO.insert(logDO);

            return seatNo;
        }

        return -1;
    }

    /**
     * 下麦位
     *
     * @param roomId
     * @param userId
     * @param isForce
     * @return
     */
    public int leaveSeat(Long roomId, Long userId, boolean isForce, String appVer) {
        return leaveSeat(roomId, getUserSeatNo(roomId, userId), isForce, appVer);
    }

    /**
     * 全麦异步下麦位
     *
     * @param roomId
     * @return
     */
    public void leaveAllAsync(Long roomId, String appVer) {
        for (int i = 1; i < 9; i++) {
            leaveSeat(roomId, i, true, appVer);
        }
    }

    /**
     * 锁定位置
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    public boolean lockSeat(Long roomId, Integer seatNo) {
        return redisService.hashPutIfAbsent(RedisRtcKeys.getRtcSeatLock(roomId), String.valueOf(seatNo), String.valueOf(System.currentTimeMillis()));
    }

    /**
     * 锁定所有位置
     *
     * @param roomId
     * @return
     */
    public int lockSeatAll(Long roomId) {
        int lockNum = 0;
        for (int i = 0; i < 9; i++) {
            if (lockSeat(roomId, i)) {
                lockNum++;
            }
        }
        return lockNum;
    }

    /**
     * 解锁位置
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    public void unlockSeat(Long roomId, Integer seatNo) {
        redisService.hashDel(RedisRtcKeys.getRtcSeatLock(roomId), String.valueOf(seatNo));
    }

    /**
     * 解锁全部麦位
     *
     * @param roomId
     * @return
     */
    public boolean unlockAllSeat(Long roomId) {
        return redisService.delKey(RedisRtcKeys.getRtcSeatLock(roomId));
    }

    /**
     * 获取麦位锁
     *
     * @param roomId
     * @return
     */
    public Set<Integer> getSeatLockSet(Long roomId) {
        Set<Object> keys = redisService.hashKeys(RedisRtcKeys.getRtcSeatLock(roomId));
        if (CollectionUtil.isNotEmpty(keys)) {
            return keys.stream().map(Convert::toInt).collect(Collectors.toSet());
        }
        return null;
    }

    /**
     * 获取麦位用户
     *
     * @param roomId
     * @return seatNo:userId
     */
    public Map<Integer, Long> getSeatUserMap(Long roomId) {
        // seatNo:userId
        Map<Object, Object> map = redisService.hashGetAll(RedisRtcKeys.getRtcRoomSeat(roomId));
        Map<Integer, Long> seatUserIds = new HashMap<>();
        for (Map.Entry<Object, Object> entry : map.entrySet()) {
            seatUserIds.put(Convert.toInt(entry.getKey()), Convert.toLong(entry.getValue()));
        }
        return seatUserIds;
    }

    public List<Long> getSeatUserIds(Long roomId) {
        Map<Object, Object> map = redisService.hashGetAll(RedisRtcKeys.getRtcRoomSeat(roomId));
        List<Long> userIds = new ArrayList<>();
        for (Map.Entry<Object, Object> entry : map.entrySet()) {
            userIds.add(Convert.toLong(entry.getValue()));
        }
        return userIds;
    }

    /**
     * 获取麦上用户数量
     *
     * @param roomId
     * @return
     */
    public int getSeatUserCount(Long roomId) {
        return Optional.ofNullable(redisService.hashSize(RedisRtcKeys.getRtcRoomSeat(roomId))).map(Convert::toInt).orElse(0);
    }

    /**
     * 获取麦位用户 id
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    public Long getSeatUserId(Long roomId, int seatNo) {
        return Optional.ofNullable(redisService.hashGet(RedisRtcKeys.getRtcRoomSeat(roomId), String.valueOf(seatNo))).map(Convert::toLong).orElse(null);
    }

    /**
     * 获取用户麦位
     *
     * @param roomId
     * @param userId
     * @return
     */
    public int getUserSeatNo(Long roomId, Long userId) {
        Map<Integer, Long> seatUserMap = getSeatUserMap(roomId);
        for (Map.Entry<Integer, Long> entry : seatUserMap.entrySet()) {
            if (userId.equals(entry.getValue())) {
                return entry.getKey();
            }
        }
        return -1;
    }

    /**
     * 设置麦位麦克风静音
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    public boolean setMicSilenced(Long roomId, int seatNo) {
        return redisService.hashPutIfAbsent(RedisRtcKeys.getRtcSeatMicSilence(roomId), String.valueOf(seatNo), String.valueOf(System.currentTimeMillis()));
    }

    /**
     * 取消麦位麦克风静音
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    public boolean cancelMicSilenced(Long roomId, int seatNo) {
        redisService.hashDel(RedisRtcKeys.getRtcSeatMicSilence(roomId), String.valueOf(seatNo));
        return true;
    }

    /**
     * 获取麦位静音状态
     *
     * @param roomId
     * @return
     */
    public Set<Integer> getSeatMicSilencedSet(Long roomId) {
        Set<Object> keys = redisService.hashKeys(RedisRtcKeys.getRtcSeatMicSilence(roomId));
        if (CollectionUtil.isNotEmpty(keys)) {
            return keys.stream().map(Convert::toInt).collect(Collectors.toSet());
        }
        return null;
    }

    /**
     * 禁用麦位麦克风
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    public boolean setMicDisabled(Long roomId, int seatNo) {
        return redisService.hashPutIfAbsent(RedisRtcKeys.getRtcSeatMicDisable(roomId), String.valueOf(seatNo), String.valueOf(System.currentTimeMillis()));
    }

    /**
     * 启用麦位麦克风
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    public boolean setMicEnabled(Long roomId, int seatNo) {
        redisService.hashDel(RedisRtcKeys.getRtcSeatMicDisable(roomId), String.valueOf(seatNo));
        return true;
    }

    /**
     * 获取麦位禁用状态
     *
     * @param roomId
     * @return
     */
    public Set<Integer> getSeatDisabledSet(Long roomId) {
        Set<Object> keys = redisService.hashKeys(RedisRtcKeys.getRtcSeatMicDisable(roomId));
        if (CollectionUtil.isNotEmpty(keys)) {
            return keys.stream().map(Convert::toInt).collect(Collectors.toSet());
        }
        return null;
    }

    /**
     * 获取用户麦位
     *
     * @param roomId
     * @param userIds
     * @return userId:seatNo
     */
    public Map<Long, Integer> getUserSeats(Long roomId, List<Long> userIds) {
        // seatNo:userId
        Map<Integer, Long> seatUsers = getSeatUserMap(roomId);
        Map<Long, Integer> userSeats = new HashMap<>();
        for (Map.Entry<Integer, Long> entry : seatUsers.entrySet()) {
            if (userIds.contains(entry.getValue())) {
                userSeats.put(entry.getValue(), entry.getKey());
            }
        }
        return userSeats;
    }

    /**
     * 设置老板席位
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    public boolean setBossSeat(Long roomId, Integer seatNo) {
        if (redisService.hashPutIfAbsent(RedisRtcKeys.getRtcBossSeat(roomId), String.valueOf(seatNo), String.valueOf(System.currentTimeMillis()))) {
            return true;
        }
        return false;
    }

    /**
     * 取消老板席位
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    public boolean removeBossSeat(Long roomId, Integer seatNo) {
        redisService.hashDel(RedisRtcKeys.getRtcBossSeat(roomId), String.valueOf(seatNo));
        return true;
    }

    /**
     * 获取老板席位
     *
     * @param roomId
     * @return
     */
    public Set<Integer> getBoosSeats(Long roomId) {
        Set<Object> keys = redisService.hashKeys(RedisRtcKeys.getRtcBossSeat(roomId));
        if (CollectionUtil.isNotEmpty(keys)) {
            return keys.stream().map(Convert::toInt).collect(Collectors.toSet());
        }
        return null;
    }

    /**
     * 创建倒计时
     *
     * @param roomId
     * @param seatNo
     * @param seconds
     * @return
     */
    public boolean newCountDown(Long roomId, Integer seatNo, Integer seconds) {
        String val = String.format("%d:%d", System.currentTimeMillis(), seconds);

        String curVal = Optional.ofNullable(redisService.hashGet(RedisRtcKeys.getRtcCountDown(roomId), String.valueOf(seatNo))).map(Object::toString).orElse("");
        if (StringUtils.isNotBlank(curVal)) {
            String[] args = curVal.split(":");
            if (System.currentTimeMillis() < Convert.toLong(args[0]) + Convert.toInt(args[1]) * 1000L) {
                return false;
            }
        }

        // 判断是否倒计时中
        redisService.hashPut(RedisRtcKeys.getRtcCountDown(roomId), String.valueOf(seatNo), val);
        return true;
    }

    /**
     * 移除倒计时
     *
     * @param roomId
     * @param seatNo
     * @return
     */
    public boolean removeCountDown(Long roomId, Integer seatNo) {
        redisService.hashDel(RedisRtcKeys.getRtcCountDown(roomId), String.valueOf(seatNo));
        return true;
    }

    /**
     * 获取麦位剩余倒计时
     *
     * @param roomId
     * @return
     */
    public Map<Integer, Integer> getCountDownRemainMap(Long roomId) {
        Map<Object, Object> countDown = redisService.hashGetAll(RedisRtcKeys.getRtcCountDown(roomId));
        if (countDown != null) {
            Map<Integer, Integer> retMap = new HashMap<>();
            for (Map.Entry<Object, Object> entry : countDown.entrySet()) {
                String[] valArr = entry.getValue().toString().split(":");
                Integer diff = Convert.toInt((System.currentTimeMillis() - Convert.toLong(valArr[0])) / 1000L);
                int remain = Convert.toInt(valArr[1]) - diff;
                if (remain > 1) {
                    retMap.put(Convert.toInt(entry.getKey()), remain);
                }
            }
            return retMap;
        }
        return null;
    }

    /**
     * 发送状态检查消息
     *
     * @param roomId
     * @param userId
     */
    public void sendCheckStatusMsg(Long roomId, Long userId, String appVer) {
        // 延时判断用户状态
        QueueRtcCheckStatusMsg rtcCheckStatusMsg = new QueueRtcCheckStatusMsg();
        rtcCheckStatusMsg.setRoomId(roomId);
        rtcCheckStatusMsg.setUserId(userId);
        rtcCheckStatusMsg.setAppVer(appVer);
        rocketProducer.sendMsg(RocketTopic.TPC_DELAY, RocketTag.RTC_CHECK_STATUS, rtcCheckStatusMsg.serialize(), System.currentTimeMillis() + TimeConstants.SECOND_30_MILLS);

        // 缓存消息时间
        redisService.hashPut(RedisRtcKeys.RTC_CHECK_TIME, getCheckKey(roomId, userId), String.valueOf(rtcCheckStatusMsg.getTimestamp()));
    }

    public static String getCheckKey(Long roomId, Long userId) {
        return String.format("%d:%d", roomId, userId);
    }

    /**
     * 判断用户是否在频道中
     *
     * @param roomId
     * @param userId
     * @return
     */
    public boolean isUserInChannel(Long roomId, Long userId) {
        AliyunConfig aliyunConfig = configService.getAliyunConfig();

        DefaultProfile profile = DefaultProfile.getProfile("cn-hangzhou", aliyunConfig.getAccessKey(), aliyunConfig.getAccessSecret());
        IAcsClient client = new DefaultAcsClient(profile);

        CommonRequest request = new CommonRequest();
        request.setSysMethod(MethodType.POST);
        request.setSysDomain("rtc.aliyuncs.com");
        request.setSysVersion("2018-01-11");
        request.setSysAction("DescribeUserInfoInChannel");
        request.putQueryParameter("RegionId", "cn-hangzhou");
        request.putQueryParameter("ChannelId", String.valueOf(roomId));
        request.putQueryParameter("UserId", String.valueOf(userId));
        request.putQueryParameter("AppId", aliyunRtcAppId);
        try {
            CommonResponse response = client.getCommonResponse(request);
            JSONObject res = JSON.parseObject(response.getData());
            return res.getBoolean("IsInChannel");
        } catch (Exception e) {
            log.error("阿里云RTC用户状态查询失败", e);
        }
        return false;
    }

    /**
     * 抢麦位
     *
     * @param roomId
     * @param seatNo
     * @param userId
     * @param skipLock
     * @return
     */
    private boolean touchSeat(Long roomId, Integer seatNo, Long userId, boolean skipLock) {
        // 判断位置是否锁定
        if (!skipLock && redisService.hashExists(RedisRtcKeys.getRtcSeatLock(roomId), String.valueOf(seatNo))) {
            return false;
        }

        // 判断用户是否在麦位
        if (getUserSeatNo(roomId, userId) > -1) {
            return false;
        }

        return redisService.hashPutIfAbsent(RedisRtcKeys.getRtcRoomSeat(roomId), String.valueOf(seatNo), String.valueOf(userId));
    }
}
