package com.starbuds.server.service.rtc.provider;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.data.SkillTypeEnum;
import com.starbuds.server.common.constant.finance.SkillOrderStatusEnum;
import com.starbuds.server.common.constant.rtc.RtcSkillOrderEnum;
import com.starbuds.server.common.pojo.db.RtcSkillOrderDO;
import com.starbuds.server.common.pojo.rtc.qo.RtcSkillOrderPageListQO;
import com.starbuds.server.service.rtc.dao.RtcSkillOrderDAO;

import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.service.rtc.dao.RtcSkillOrderReadHisDAO;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;
import static com.starbuds.server.service.rtc.util.SpringContextUtil.isDev;

import org.springframework.stereotype.Component;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 派单订单表
 */
@Component
@Slf4j
public class RtcSkillOrderProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "orderId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private RtcSkillOrderDAO rtcSkillOrderDAO;

    @Resource
    private RtcSkillOrderReadHisDAO rtcSkillOrderReadHisDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public RtcSkillOrderDO create(RtcSkillOrderDO po) {
        po.setOrderId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        boolean flag = rtcSkillOrderDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("派单订单表创建成功 {}", json);
            return po;
        }
        log.warn("派单订单表创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<RtcSkillOrderDO> list) {
        boolean flag = rtcSkillOrderDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(RtcSkillOrderDO po) {
        boolean flag = rtcSkillOrderDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(RtcSkillOrderDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getOrderId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = rtcSkillOrderDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public RtcSkillOrderDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return rtcSkillOrderDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return rtcSkillOrderDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<RtcSkillOrderDO> getPageList(RtcSkillOrderPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<RtcSkillOrderDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(rtcSkillOrderDAO.selectPageList(qw));
        page.setTotal(rtcSkillOrderDAO.selectCount(qw));
        return page;
    }

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    public Map<Long, RtcSkillOrderDO> getMapByIds(Collection ids) {
        return rtcSkillOrderDAO.selectMapByIds(ids);
    }

    // 手动编码开始

    /**
     * 获取房间内订单数量
     *
     * @param roomId
     * @return
     */
    public Integer getOrderInRoomCount(Long roomId) {
        return Optional.ofNullable(rtcSkillOrderDAO.selectOrderInRoomCount(roomId)).orElse(0);
    }

    /**
     * 获取大厅订单数量
     *
     * @return
     */
    public Integer getOrderCreatedCount(Long userId) {
        List<RtcSkillOrderDO> userOrderList = rtcSkillOrderDAO.selectOrderCreatedListByUserId(userId);
        return getUnreadOrderCount(userOrderList, userId);
    }

    /**
     * 获取最新发布的订单
     *
     * @return
     */
    public RtcSkillOrderDO getLastOrderCreated() {
        return rtcSkillOrderDAO.selectLastOrderCreated();
    }

    /**
     * 获取大厅订单数量
     *
     * @return
     */
    public Integer getAllOrderCreatedCount(Long userId) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("status", RtcSkillOrderEnum.WAIT.getValue());
        List<RtcSkillOrderDO> orderList = rtcSkillOrderDAO.selectList(qw);
        return getUnreadOrderCount(orderList, userId);
    }

    /**
     * 获取用户的未读派单数
     *
     * @param list
     * @param userId
     * @return
     */
    private Integer getUnreadOrderCount(List<RtcSkillOrderDO> list, Long userId) {
        if (CollectionUtil.isEmpty(list)) {
            return 0;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq("userId", userId);
        qw.in("orderId", list.stream().map(RtcSkillOrderDO::getOrderId).collect(Collectors.toList()));
        Integer readHisCount = Optional.ofNullable(rtcSkillOrderReadHisDAO.selectCount(qw)).orElse(0);
        if (isDev()) {
            log.info("获取大厅未读数量3: {}, userId: {}", list.size() - readHisCount, userId);
        }
        return list.size() - readHisCount;
    }

    /**
     * 获取大厅某个技能的订单数量
     *
     * @return
     */
    public Integer getOrderCreatedCountBySkill(Long skillId) {
        return Optional.ofNullable(rtcSkillOrderDAO.selectOrderCreatedCountBySkill(skillId)).orElse(0);
    }

    /**
     * 获取大厅某个技能的订单列表
     *
     * @return
     */
    public List<RtcSkillOrderDO> getOrderCreatedListBySkill(Long skillId) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("status", RtcSkillOrderEnum.WAIT.getValue());
        qw.eq("skillId", skillId);

        return rtcSkillOrderDAO.selectList(qw);
    }

    /**
     * 查询待接单的订单+已完成的订单 总共100条
     */
    public PageList<RtcSkillOrderDO> getListInStatus(RtcSkillOrderPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<RtcSkillOrderDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        // 利用createTime限制100条数量
        Long endTime = rtcSkillOrderDAO.selectListInStatusLimit();
        qw.in("status", Arrays.asList(RtcSkillOrderEnum.WAIT.getValue(), RtcSkillOrderEnum.FINISH.getValue()));
        if (endTime != null) {
            qw.ge("createTime", endTime);
        }
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(rtcSkillOrderDAO.selectListInStatus(qw));
        page.setTotal(rtcSkillOrderDAO.selectListInStatusCount(qw));

        return page;
    }
}