package com.starbuds.server.service.rtc.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.db.WishDO;
import com.starbuds.server.service.rtc.dao.WishDAO;
import com.starbuds.server.common.pojo.rtc.qo.WishPageListQO;

import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.api.PageList;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Component;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 许愿表-由管理后台配置
 */
@Component
@Slf4j
public class WishProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "wishId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private WishDAO wishDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public WishDO create(WishDO po) {
        po.setWishId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        po.setIsDeleted(BoolIntVal.False);
        boolean flag = wishDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("许愿表-由管理后台配置创建成功 {}", json);
            return po;
        }
        log.warn("许愿表-由管理后台配置创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<WishDO> list) {
        boolean flag = wishDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(WishDO po) {
        boolean flag = wishDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(WishDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getWishId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = wishDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public WishDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return wishDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return wishDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<WishDO> getPageList(WishPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<WishDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(wishDAO.selectPageList(qw));
        page.setTotal(wishDAO.selectCount(qw));
        return page;
    }

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    public Map<Long, WishDO> getMapByIds(Collection ids) {
        return wishDAO.selectMapByIds(ids);
    }

    // 手动编码开始

    /**
     * 获取所有可用的许愿列表
     */
    public List<WishDO> getAvailableList() {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("status", BoolIntVal.True);
        qw.eq("isDeleted", BoolIntVal.False);
        return wishDAO.selectList(qw);
    }


}