package com.starbuds.server.service.rtc.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.rtc.WishStatusEnum;
import com.starbuds.server.common.pojo.db.WishUserTaskDO;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.rtc.dao.WishUserTaskDAO;
import com.starbuds.server.common.pojo.rtc.qo.WishUserTaskPageListQO;

import com.starbuds.server.common.pojo.api.PageList;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Component;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 许愿用户任务表，用户打开许愿任务时创建当天记录
 */
@Component
@Slf4j
public class WishUserTaskProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "wishTaskId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private WishUserTaskDAO wishUserTaskDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public WishUserTaskDO create(WishUserTaskDO po) {
        po.setWishTaskId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        boolean flag = wishUserTaskDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("许愿用户任务表，用户打开许愿任务时创建当天记录创建成功 {}", json);
            return po;
        }
        log.warn("许愿用户任务表，用户打开许愿任务时创建当天记录创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<WishUserTaskDO> list) {
        boolean flag = wishUserTaskDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(WishUserTaskDO po) {
        boolean flag = wishUserTaskDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(WishUserTaskDO po) {
        po.setCreateTime(null);

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getWishTaskId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = wishUserTaskDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public WishUserTaskDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return wishUserTaskDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return wishUserTaskDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<WishUserTaskDO> getPageList(WishUserTaskPageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<WishUserTaskDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(wishUserTaskDAO.selectPageList(qw));
        page.setTotal(wishUserTaskDAO.selectCount(qw));
        return page;
    }

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    public Map<Long, WishUserTaskDO> getMapByIds(Collection ids) {
        return wishUserTaskDAO.selectMapByIds(ids);
    }

    // 手动编码开始

    /**
     * 查询用户当天许愿任务
     */
    public List<WishUserTaskDO> getUserCurrentDayList(Long userId) {
        Long dayStart = DateUtil.getDayBegin();
        return wishUserTaskDAO.selectUserCurrentDayList(userId, dayStart);
    }

    /**
     * 更新许愿任务状态
     */
    public boolean updateWishTaskStatus(Long wishTaskId, Long userId, Integer status) {
        return wishUserTaskDAO.updateWishTaskStatus(wishTaskId, userId, status);
    }

    /**
     * 更新许愿任务开启状态
     */
    public boolean updateWishTaskOpenStatus(Long wishTaskId, Long userId, Long openTime) {
        return wishUserTaskDAO.updateWishTaskOpenStatus(wishTaskId, userId, openTime);
    }

    /**
     * 更新许愿任务状态
     */
    public WishUserTaskDO getOpenedWishTask(Long userId) {
        Long dayStart = DateUtil.getDayBegin();

        QueryWrapper qw = new QueryWrapper();
        qw.eq("userId", userId);
        qw.eq("status", WishStatusEnum.OPEN.getValue());
        qw.ge("createTime", dayStart);
        qw.eq("isDeleted", 0);

        return wishUserTaskDAO.selectOne(qw);
    }

}